//	Application.h - Class handling the various application bits and pieces.
//  ----------------------------------------------------------------------------
//	This file is part of 'Tile Massacre Shmup', my entry to the TIGSource
//	Assemblee contest.
//	Copyright (C) 2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef APPLICATION_H_
#define APPLICATION_H_

#include "NiallsDrawingLib/Drawer.h"
#include "PortAudioDevice.h"
#include "sound/TitleMusic.h"
#include "sound/ShooterMusic.h"
#include "sound/GauntletMusic.h"

#include "SDL.h"
#include <vector>
#include <string>
#include <map>

class TileSet;
class GameState;
typedef std::vector<std::string> CmdLineParams;

//------------------------------------------------------------------------------
///	Struct representing the player's high score for a level.
struct LevelScore
{
	///	The number of crates collected.
	int crates;
	///	The number of reflectors used.
	int reflectors;
	///	The time taken.
	float time;
};

//------------------------------------------------------------------------------
///	Class handling the various application bits and pieces.
class Application : public PortAudioCallback
{
  public:
	///	Constructor.
	/*!
		\param commandLine A vector containing all the command line parameters
		passed in.

		This will call SDL_Init() with the SDL_INIT_VIDEO and SDL_INIT_TIMER
		flags.  It will also open a window according to the command line
		arguments (e.g. fullscreen/windowed, geometry, bit depth etc.).
	 */
	Application(const CmdLineParams &commandLine);
	///	Destructor.
	/*!
		Will call SDL_Quit().
	 */
	~Application();

	///	Where the sound is generated and sent to the soundcard.
	void getAudio(float *input, float *output, int numSamples);

	///	The main event loop occurs inside this method.
	void eventLoop();

	///	Quits the game.
	void quit();

	///	Moves to the next game state.
	void nextState(const std::string newState, uint8_t *level = 0, TileSet *tiles = 0);
  private:
	///	Called when a key down event is received.
	void keyDown(const SDL_keysym& keyPressed);
	///	Called when a key up event is received.
	void keyUp(const SDL_keysym& keyPressed);

	///	Called when a mouse down event is received.
	void mouseDown(int button, int posX, int posY);
	///	Called when a mouse up event is received.
	void mouseUp(int button, int posX, int posY);
	///	Called when a mouse move event is received.
	void mouseMove(int posX, int posY);

	///	Called when a joystick y axis changes.
	void joyYMove(bool dir);
	///	Called when a joystick x axis changes.
	void joyXMove(bool dir);
	///	Called when a joystick button is pressed.
	void joyDown(int button);
	///	Called when a joystick button is released.
	void joyUp(int button);

	///	Loads the high scores file.
	void loadHighScores();
	///	Saves the high scores file.
	void saveHighScores();

	///	Helper method to create a jpeg file.
	void writeJpeg(unsigned char *pixels, const std::string& filename);

	///	Helper method to load drawer's glyphs from Fonts.cpp.
	void loadGlyphs();

	///	Our PortAudioDevice.
	PortAudioDevice portAudioDevice;

	///	We set this to true when it's time to quit the program.
	bool timeToQuit;
	///	We use this to determine if we need to repaint the screen immediately.
	Uint32 lastRepaint;

	///	The width of the window.
	int width;
	///	The height of the window.
	int height;
	///	True if we're not playing any sound.
	bool noSound;
	///	True if we're not playing the background music.
	bool noMusic;
	///	The path to files we may be rendering offline.
	std::string offlineFilename;
	///	The number of seconds we're going to render offline for.
	unsigned int offlineTime;

	///	For playing with a joystick instead of the keyboard.
	SDL_Joystick *joystick;
	///	Used to avoid 'stickiness' issues.
	bool joyUpIsDown;
	///	Used to avoid 'stickiness' issues.
	bool joyDownIsDown;
	///	Used to avoid 'stickiness' issues.
	bool joyLeftIsDown;
	///	Used to avoid 'stickiness' issues.
	bool joyRightIsDown;

	///	Draws everything on screen.
	Drawer *drawer;
	///	Plays the background music.
	TitleMusic titleMusic;
	///	Plays the background music.
	ShooterMusic shooterMusic;
	///	Plays the background music.
	GauntletMusic gauntletMusic;

	///	The current game state.
	GameState *state;
	///	Which level the player's currently on.
	int currentLevel;

	///	The top 3 high scores.
	int highScores[6];
	///	The top 3 high scores.
	std::string highScoreNames[6];
};

#endif
