//	FilePath.cpp - A class representing a file path on the OS.
//	----------------------------------------------------------------------------
//	This file is part of 'CavernsOfLight', a game.
//	Copyright (c) 2009 Niall Moody.
//
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "FilePath.h"

#ifdef WIN32

#include <windows.h>
#include <shlobj.h>

#else

#include <errno.h>
#include <list>
#include <cstdlib>
#include <dirent.h>
#include <sys/stat.h>

#endif

#ifdef OSX

#include "SDL.h"
#include <CoreServices/CoreServices.h>
#include <CoreFoundation/CoreFoundation.h>

#endif

#include <sstream>
#include <iostream>

using namespace std;

//------------------------------------------------------------------------------
FilePath::FilePath()
{
	
}

//------------------------------------------------------------------------------
FilePath::FilePath(const string& p):
path(p)
{
	
}

//------------------------------------------------------------------------------
FilePath::FilePath(const FilePath& other):
path(other.path)
{
	
}

//------------------------------------------------------------------------------
FilePath::~FilePath()
{
	
}

//------------------------------------------------------------------------------
void FilePath::setPath(const std::string& p)
{
	path = p;
}

//------------------------------------------------------------------------------
string FilePath::getExtension() const
{
	string retval;

	if(path.empty())
		return retval;
	else if(isDirectory())
		return retval;

	retval = path.substr(path.find_last_of('.')+1);

	return retval;
}

//------------------------------------------------------------------------------
string FilePath::getName() const
{
	string retval;

#ifdef WIN32
	retval = path.substr(path.find_last_of('\\')+1);
#else
	retval = path.substr(path.find_last_of('/')+1);
#endif
	retval = retval.substr(0, retval.find_last_of('.'));

	return retval;
}

//------------------------------------------------------------------------------
string FilePath::getNameWithExtension() const
{
	string retval;

#ifdef WIN32
	retval = path.substr(path.find_last_of('\\')+1);
#else
	retval = path.substr(path.find_last_of('/')+1);
#endif

	return retval;
}

//------------------------------------------------------------------------------
bool FilePath::exists() const
{
	bool retval = false;

	if(path.empty())
		return retval;

#ifdef WIN32
	DWORD result;

	result = GetFileAttributes(path.c_str());
	retval = (result != 0xFFFFFFFF);
#else
	struct stat fileInfo;

	if(stat(path.c_str(), &fileInfo) == 0)
		retval = true;
#endif

	return retval;
}

//------------------------------------------------------------------------------
bool FilePath::isFile() const
{
	bool retval = false;

	if(path.empty())
		return retval;

#ifdef WIN32
	DWORD result;

	result = GetFileAttributes(path.c_str());
	retval = (result != 0xFFFFFFFF) && ((result&FILE_ATTRIBUTE_DIRECTORY) == 0);
#else
	struct stat fileInfo;

	//Check it exists first.
	if(stat(path.c_str(), &fileInfo) == 0)
	{
		DIR *d;

		//Now check whether it's a directory or a file.
		d = opendir(path.c_str());
		if(d)
			closedir(d);
		else
			retval = true;
	}
#endif

	return retval;
}

//------------------------------------------------------------------------------
bool FilePath::isDirectory() const
{
	bool retval = false;

	if(path.empty())
		return retval;

#ifdef WIN32
	DWORD result;

	result = GetFileAttributes(path.c_str());
	retval = (result != 0xFFFFFFFF) && ((result&FILE_ATTRIBUTE_DIRECTORY) != 0);

	if(result == 0xFFFFFFFF)
	{
		char tempstr[MAX_PATH];

		FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM | 
					  FORMAT_MESSAGE_IGNORE_INSERTS,
					  NULL,
					  GetLastError(),
					  MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
					  tempstr,
					  MAX_PATH,
					  NULL);
		cout << "FilePath::isDirectory() error: " << tempstr << endl;
	}
#else
	struct stat fileInfo;

	//Check it exists first.
	if(stat(path.c_str(), &fileInfo) == 0)
	{
		DIR *d;

		//Now check whether it's a directory or a file.
		d = opendir(path.c_str());
		if(d)
		{
			closedir(d);
			retval = true;
		}
	}
#endif

	return retval;
}

//------------------------------------------------------------------------------
bool FilePath::createAsDirectory() const
{
	bool retval = false;

#ifdef WIN32
	retval = CreateDirectory(path.c_str(), 0) != 0;
#else
	retval = (mkdir(path.c_str(), S_IRWXU|S_IRWXG|S_IROTH|S_IXOTH) == 0);
#endif

	return retval;
}

//------------------------------------------------------------------------------
int FilePath::getNumChildren() const
{
	int retval = 0;

	if(path.empty())
		return retval;
	else if(!isDirectory())
		return retval;

#ifdef WIN32
	HANDLE handle;
	stringstream tempPath;
	WIN32_FIND_DATA findData;

	//We do a search for all the files in this directory.
	tempPath << path << "\\*";

	handle = FindFirstFile(tempPath.str().c_str(), &findData);
	if(handle != INVALID_HANDLE_VALUE)
	{
		if((string(findData.cFileName) != ".") &&
		   (string(findData.cFileName) != ".."))
		{
			++retval;
		}

		while(FindNextFile(handle, &findData))
		{
			if((string(findData.cFileName) != ".") &&
			   (string(findData.cFileName) != ".."))
			{
				++retval;
			}
		}

		FindClose(handle);
	}
#else
	DIR *d;
	struct dirent *entry;

	d = opendir(path.c_str());
	if(d)
	{
		while(entry = readdir(d))
		{
			if((string(entry->d_name) != ".") &&
			   (string(entry->d_name) != ".."))
			{
				++retval;
			}
		}

		closedir(d);
	}
#endif

	return retval;
}

//------------------------------------------------------------------------------
FilePath FilePath::getChild(int index) const
{
	int count = -1;
	FilePath retval;
	stringstream tempPath;

#ifdef WIN32
	HANDLE handle;
	WIN32_FIND_DATA findData;

	//We do a search for all the files in this directory.
	tempPath << path << "\\*";

	handle = FindFirstFile(tempPath.str().c_str(), &findData);
	if(handle != INVALID_HANDLE_VALUE)
	{
		if((string(findData.cFileName) != ".") &&
		   (string(findData.cFileName) != ".."))
		{
			++count;
		}

		if(count == index)
		{
			tempPath.str("");
			tempPath << path << "\\" << findData.cFileName;
			retval.setPath(tempPath.str());
		}

		while(FindNextFile(handle, &findData))
		{
			if((string(findData.cFileName) != ".") &&
			   (string(findData.cFileName) != ".."))
			{
				++count;
			}

			if(count == index)
			{
				tempPath.str("");
				tempPath << path << "\\" << findData.cFileName;
				retval.setPath(tempPath.str());
				break;
			}
		}

		FindClose(handle);
	}
#else
	DIR *d;
	struct dirent *entry;
	list<string> orderedChildren;
	list<string>::iterator it;

	d = opendir(path.c_str());
	if(d)
	{
		while(entry = readdir(d))
		{
			//readdir() does not return its entries in the correct order, so...
			if((string(entry->d_name) != ".") &&
			   (string(entry->d_name) != ".."))
			{
				orderedChildren.push_back(entry->d_name);
			}
		}
		orderedChildren.sort();

		for(it=orderedChildren.begin();it!=orderedChildren.end();++it)
		{
			++count;
			if(count == index)
			{
				tempPath.str("");
				tempPath << path << "/" << *it;
				retval.setPath(tempPath.str());
				break;
			}
		}

		closedir(d);
	}
#endif

	return retval;
}

//------------------------------------------------------------------------------
FilePath FilePath::getChild(const string& child) const
{
	FilePath retval;
	stringstream tempPath;

#ifdef WIN32
	tempPath << path << "\\" << child;
#else
	tempPath << path << "/" << child;
#endif
	retval.setPath(tempPath.str());

	return retval;
}

//------------------------------------------------------------------------------
FilePath FilePath::getParent() const
{
	FilePath retval;
	string tempstr;

#ifdef WIN32
	tempstr = path.substr(0, path.find_last_of('\\'));
#else
	tempstr = path.substr(0, path.find_last_of('/'));
#endif
	retval.setPath(tempstr);

	return retval;
}

//------------------------------------------------------------------------------
bool FilePath::copyTo(const string& dest)
{
	bool retval = false;

	if(path.empty())
		return retval;

#ifdef WIN32
	retval = (CopyFile(path.c_str(), dest.c_str(), false) != 0);
#else
	stringstream tempstr;

	tempstr << "cp " << path << " " << dest;
	retval = (system(tempstr.str().c_str()) == 0);
#endif

	return retval;
}

//------------------------------------------------------------------------------
FilePath FilePath::getAppDataPath()
{
	FilePath retval;

#ifdef WIN32
	char tempstr[MAX_PATH];

	if(SHGetFolderPath(NULL, CSIDL_APPDATA, NULL, 0, tempstr) == S_OK)
		retval.setPath(tempstr);
#elif defined(LINUX)
	char *d = getenv("HOME");
	stringstream tempstr;

	if(d)
		tempstr << d << "/." << APP_NAME;
	else
		tempstr << "~/." << APP_NAME; //Will this even work?
	retval.setPath(tempstr.str());
#elif defined(OSX)
	char *d = getenv("HOME");
	stringstream tempstr;

	if(d)
		tempstr << d << "/Library/Application Support";
	else
		tempstr << "~/Library/Application Support"; //Will this even work?
	retval.setPath(tempstr.str());
#endif

	return retval;
}

//------------------------------------------------------------------------------
FilePath FilePath::getCurrentExecutableDir()
{
	FilePath retval;

#ifdef WIN32
	string tempstr2;
	char tempstr[MAX_PATH];

	if(GetModuleFileName(NULL, tempstr, MAX_PATH))
	{
		tempstr2 = tempstr;

		retval.setPath(tempstr2.substr(0, tempstr2.find_last_of("\\")));
	}
#elif defined(LINUX)
	stringstream tempstr;

	tempstr << INSTALL_DIR << "/bin";
	retval.setPath(tempstr.str());
#elif defined(OSX)
	FSRef fsRef;
	CFURLRef bundlePath;

	bundlePath = CFBundleCopyBundleURL(CFBundleGetMainBundle());

	if(CFURLGetFSRef(bundlePath, &fsRef))
	{
		char tempstr[2048];
		FilePath tempPath;

		FSRefMakePath(&fsRef, (Uint8 *)tempstr, 2048);

		tempPath.setPath(tempstr);
		retval = tempPath.getParent().getPath();
	}
#endif

	return retval;
}

//------------------------------------------------------------------------------
FilePath FilePath::getDataFilesDir()
{
	FilePath retval;

#ifdef WIN32
	string tempstr2;
	char tempstr[MAX_PATH];

	if(GetModuleFileName(NULL, tempstr, MAX_PATH))
	{
		tempstr2 = tempstr;

		retval.setPath(tempstr2.substr(0, tempstr2.find_last_of("\\")));
	}
#elif defined(LINUX)
	stringstream tempstr;

	tempstr << INSTALL_DIR << "/share/" << APP_NAME;
	retval.setPath(tempstr.str());
#elif defined(OSX)
	FSRef fsRef;
	CFURLRef bundlePath;
	stringstream tempStream;

	bundlePath = CFBundleCopyBundleURL(CFBundleGetMainBundle());

	if(CFURLGetFSRef(bundlePath, &fsRef))
	{
		char tempstr[2048];

		FSRefMakePath(&fsRef, (Uint8 *)tempstr, 2048);

		tempStream << tempstr << "/Contents/Resources";
		retval = tempStream.str();
	}
#endif

	return retval;
}
