//	Sounds.cpp - Singleton to play sound effects.
//  ----------------------------------------------------------------------------
//	This file is part of 'Die, you Stupid Hurdlers!', a game for Ludum Dare 14.
//	Copyright (C) 2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "Sounds.h"
#include "FilePath.h"

#include <cassert>
#include <sstream>
#include <fstream>
#include <iostream>
#include <vorbis/codec.h>

using namespace std;

//------------------------------------------------------------------------------
Sounds& Sounds::getInstance()
{
	static Sounds retval;

	return retval;
}

//------------------------------------------------------------------------------
void Sounds::playSound(SoundType sound)
{
	int i;

	for(i=0;i<NumSounds;++i)
	{
		if(!sounds[i].active)
		{
			assert(soundFiles[sound].data != 0);
			sounds[i].data = soundFiles[sound].data;
			sounds[i].size = soundFiles[sound].size;
			sounds[i].index = 0;
			sounds[i].active = true;
			++activeSounds;
			break;
		}
	}
}

//------------------------------------------------------------------------------
void Sounds::getAudio(float *buffer, int numSamples)
{
	int i, j;
	int remainingSounds = activeSounds;

	if(activeSounds == 0)
		return;

	for(j=0;j<NumSounds;++j)
	{
		if(sounds[j].active)
		{
			for(i=0;i<numSamples;++i)
			{
				buffer[(i*2)] += sounds[j].data[sounds[j].index];// * 0.5f;
				buffer[(i*2)+1] += sounds[j].data[sounds[j].index];// * 0.5f;
				++sounds[j].index;
				if(sounds[j].index >= sounds[j].size)
				{
					sounds[j].active = false;
					break;
				}
			}

			--remainingSounds;
			if(remainingSounds == 0)
				break;
		}
	}
}

//------------------------------------------------------------------------------
Sounds::Sounds():
activeSounds(0)
{
	FilePath tempPath;

	//Ship Gun.
	tempPath = FilePath::getDataFilesDir().getChild("sounds").getChild("Shaktool-yoink.ogg");
	loadFile(tempPath.getPath(), &(soundFiles[ShipGun]));

	//Tile Hit.
	tempPath = FilePath::getDataFilesDir().getChild("sounds").getChild("Boomlinde-dodile.ogg");
	loadFile(tempPath.getPath(), &(soundFiles[TileHit]));

	//Ship Die.
	tempPath = FilePath::getDataFilesDir().getChild("sounds").getChild("Shaktool-explode2.ogg");
	loadFile(tempPath.getPath(), &(soundFiles[ShipDie]));

	//Ship New.
	tempPath = FilePath::getDataFilesDir().getChild("sounds").getChild("Boomlinde-bupss.ogg");
	loadFile(tempPath.getPath(), &(soundFiles[ShipNew]));

	//Enemy Shoot.
	tempPath = FilePath::getDataFilesDir().getChild("sounds").getChild("Shaktool-laser4.ogg");
	loadFile(tempPath.getPath(), &(soundFiles[EnemyShoot]));

	//Gauntlet Gun.
	tempPath = FilePath::getDataFilesDir().getChild("sounds").getChild("Atuun-MachineGun1.ogg");
	loadFile(tempPath.getPath(), &(soundFiles[GauntletGun]));
	//Tile Explode.
	tempPath = FilePath::getDataFilesDir().getChild("sounds").getChild("Boomlinde-kwaou.ogg");
	loadFile(tempPath.getPath(), &(soundFiles[TileExplode]));

	//Skeleton Die.
	tempPath = FilePath::getDataFilesDir().getChild("sounds").getChild("Boomlinde-GlonkEcho.ogg");
	loadFile(tempPath.getPath(), &(soundFiles[SkeletonDie]));

	//Pickup.
	tempPath = FilePath::getDataFilesDir().getChild("sounds").getChild("Boomlinde-brip.ogg");
	loadFile(tempPath.getPath(), &(soundFiles[Pickup]));

	//Gauntlet Die.
	tempPath = FilePath::getDataFilesDir().getChild("sounds").getChild("NiallM-Ow1.ogg");
	loadFile(tempPath.getPath(), &(soundFiles[GauntletDie]));

	//Gauntlet New.
	tempPath = FilePath::getDataFilesDir().getChild("sounds").getChild("Boomlinde-ziing.ogg");
	loadFile(tempPath.getPath(), &(soundFiles[GauntletNew]));

	//Bullet Hit.
	tempPath = FilePath::getDataFilesDir().getChild("sounds").getChild("Shaktool-switch.ogg");
	loadFile(tempPath.getPath(), &(soundFiles[BulletHit]));
}

//------------------------------------------------------------------------------
Sounds::~Sounds()
{
	int i;

	for(i=0;i<NumSoundTypes;++i)
		delete [] soundFiles[i].data;
}

//------------------------------------------------------------------------------
void Sounds::loadFile(const string& tempstr, SoundFile *sf)
{
	FILE *file;
	OggVorbis_File vorbisFile;

	int i;
	int result;
	float **tempBuf;
	int currentBitstream = 0;
	unsigned int samplesRead = 0;
	unsigned int length;
	unsigned int runningCount = 0;

	file = fopen(tempstr.c_str(), "rb");
	if(!file)
		cout << "vorbisfile: fopen could not open file: " << tempstr << endl;
	else
	{
		if(ov_open(file, &vorbisFile, NULL, 0) < 0)
			cout << "vorbisfile: Could not open file: " << tempstr << endl;
		else
		{
			if(ov_streams(&vorbisFile) > 0)
			{
				cout << "vorbisfile: Loaded file: " << tempstr << endl;

				length = ov_pcm_total(&vorbisFile, 0);
				sf->size = length;
				sf->data = new float[sf->size];

				while(runningCount < sf->size)
				{
					result = ov_read_float(&vorbisFile,
										   &tempBuf,
										   sf->size,
										   0);
					if(result > 0)
					{
						for(i=0;i<result;++i)
							sf->data[runningCount+i] = tempBuf[0][i];
					}

					runningCount += result;
					if(result == 0)
						break;
				}
			}
			else
			{
				ov_clear(&vorbisFile);
				cout << "vorbisfile: There are no streams in this file?" << endl;
			}
		}
	}
}
