//	FinalGauntletGamePlay.cpp - The gauntlet stage.
//  ----------------------------------------------------------------------------
//	This file is part of 'Tile Massacre Shmup', my entry to the TIGSource
//	Assemblee contest.
//	Copyright (C) 2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifdef DEBUG
#pragma warning(disable : 4786) //Stupid MS headers...
#endif

#include "FinalGauntletGamePlay.h"
#include "NiallsDrawingLib/Drawer.h"
#include "NiallsDrawingLib/ImageLoader.h"
#include "Images.h"
#include "TileSet.h"
#include "ShooterEnemy.h"
#include "Explosion.h"
#include "EnemySpawner.h"
#include "GauntletBullet.h"
#include "GameCounter.h"
#include "Random.h"
#include "HealthPickup.h"
#include "BulletPickup.h"
#include "sound/Sounds.h"
#include "ShipBullet.h"
#include "Skeleton.h"
#include "DebugStatement.h"

#include <cmath>
#include <cassert>
#include <sstream>

using namespace std;

//------------------------------------------------------------------------------
uint32_t FinalGauntletGamePlay::heartImage = 0;
uint32_t FinalGauntletGamePlay::particle1 = 0;
uint32_t FinalGauntletGamePlay::particle2 = 0;

//------------------------------------------------------------------------------
FinalGauntletGamePlay::FinalGauntletGamePlay(Application *app, uint8_t *level, TileSet *tiles):
GameState(app),
tileSet(tiles),
arrowBounce(0.0f),
fireIsDown(false),
bulletTimer(0),
pacifist(true),
shake(1.0f),
gameOver(false),
levelComplete(false),
paused(false)
{
	int i;
	int x, y;
	int numGround, numWall, numImpassable;

	//Initialise Skeleton timer.
	skeletonTimer = (int)(Random::getGlobal().getRandomFloat() * 60.0f) + 300.0f;

	numGround = tileSet->getNumGroundTiles();
	numWall = tileSet->getNumWallTiles();
	numImpassable = tileSet->getNumImpassableTiles();

	//Create the map.
	for(i=0;i<(MapWidth*MapHeight);++i)
	{
		map[i] = level[i];
		walls[i] = 32;

		if(map[i] < 63)
		{
			assert(map[i] < numGround);
			if(map[i] >= numGround)
			{
				DebugStatement("Ground tile out of range:");
				DebugStatement(map[i]);
				map[i] = 0;
			}
		}
		else if(map[i] < 128)
		{
			assert((map[i]-64) < numWall);
			if((map[i]-64) >= numWall)
			{
				DebugStatement("Wall tile out of range:");
				DebugStatement(map[i]);
				map[i] = 0;
			}
		}
		else if(map[i] < 192)
		{
			assert((map[i]-128) < numImpassable);
			if((map[i]-128) >= numImpassable)
			{
				DebugStatement("Impassable tile out of range:");
				DebugStatement(map[i]);
				map[i] = 0;
			}
		}
		else
		{
			assert(map[i] == 192);
			if(map[i] > 192)
			{
				DebugStatement("Spawner tile out of range:");
				DebugStatement(map[i]);
				map[i] = 0;
			}
		}
	}

	//Add the enemies.
	for(i=0;i<4;++i)
		enemies.push_back(new ShooterEnemy());

	//Make sure the entrance to the level is clear.
	if(map[(9*MapWidth)+3] > 63)
		map[(9*MapWidth)+3] = 0;

	//Add pickups to all the wall tiles.
	for(y=0;y<MapHeight;++y)
	{
		for(x=0;x<MapWidth;++x)
		{
			if((map[(y*MapWidth)+x] > 63) && (map[(y*MapWidth)+x] < 128))
			{
				if(Random::getGlobal().getRandomFloat() > 0.5f)
					pickups.push_back(new HealthPickup((x*32), (y*32)));
				else
					pickups.push_back(new BulletPickup((x*32), (y*32)));
			}
		}
	}

	//Setup the particles.
	for(i=0;i<NumParticles;++i)
	{
		particles[i].setDensity(0.1f);
		particles[i].setDirection(0.1f, -0.2f);
		particles[i].setLifespan(3.0f);
		particles[i].setSize(6.0f, 6.0f);
		particles[i].setRandomSize(true);
		if(particle1 != 0)
		{
			particles[i].addImage(particle1);
			particles[i].addImage(particle2);
		}
	}
}

//------------------------------------------------------------------------------
FinalGauntletGamePlay::~FinalGauntletGamePlay()
{
	int i;

	for(i=0;i<bullets.size();++i)
		delete bullets[i];
	for(i=0;i<enemyBullets.size();++i)
		delete enemyBullets[i];
	for(i=0;i<explosions.size();++i)
		delete explosions[i];
	for(i=0;i<enemies.size();++i)
		delete enemies[i];
	for(i=0;i<pickups.size();++i)
		delete pickups[i];
	for(i=0;i<skeletons.size();++i)
		delete skeletons[i];
}

//------------------------------------------------------------------------------
void FinalGauntletGamePlay::draw(Drawer *d)
{
	int i;
	int x, y;
	int tempint;
	float tileScale = 2.0f;
	float shakeOffset = sinf(shake * 4.0f * 3.14159265f) * 4.0f;

	if(!heartImage)
	{
		ImageLoader imLoad;
		ImageData data;

		data = imLoad.loadImage(Images::heart_png, Images::heart_pngSize);
		heartImage = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::particle1_png, Images::particle1_pngSize);
		particle1 = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::particle2_png, Images::particle2_pngSize);
		particle2 = d->loadImage(data.data, data.width, data.height);

		for(i=0;i<NumParticles;++i)
		{
			particles[i].addImage(particle1);
			particles[i].addImage(particle2);
		}
	}

	if(shake < 1.0f)
		shake += 0.1f;

	//Set the tile scale factor to account for tiles that aren't 16x16.
	if(tileSet->getTileSize() != 16)
		tileScale = 32.0f/(float)tileSet->getTileSize();

	if(!gameOver && !levelComplete && !paused)
	{
		//Fire a new bullet if necessary.
		if(fireIsDown && !bulletTimer)
		{
			bool big = (player.getNumBigBullets() > 0);
			float xDir, yDir;

			bulletTimer = 5;

			if(player.getLeft())
				xDir = -1.0f;
			else if(player.getRight())
				xDir = 1.0f;
			else
				xDir = 0.0f;
			if(player.getUp())
				yDir = -1.0f;
			else if(player.getDown())
				yDir = 1.0f;
			else
				yDir = 0.0f;
			if((xDir == 0.0f) && (yDir == 0.0f))
			{
				switch(player.getImage())
				{
					case 0:
						xDir = 0.0f;
						yDir = -1.0f;
						break;
					case 1:
						xDir = 1.0f;
						yDir = 0.0f;
						break;
					case 2:
						xDir = 0.0f;
						yDir = 1.0f;
						break;
					case 3:
						xDir = -1.0f;
						yDir = 0.0f;
						break;
				}
			}
			bullets.push_back(new GauntletBullet(player.getX()+8.0f,
												 player.getY()+8.0f,
												 xDir,
												 yDir,
												 big));
			if(big)
				player.decrementBigBullets();
			pacifist = false;

			Sounds::getInstance().playSound(Sounds::GauntletGun);
		}
		else if(bulletTimer > 0)
			--bulletTimer;

		//Move the bullets.
		for(i=(bullets.size()-1);i>=0;--i)
		{
			if(bullets[i]->move())
			{
				delete bullets[i];
				bullets.erase(bullets.begin()+i);
			}
		}
		for(i=(enemyBullets.size()-1);i>=0;--i)
		{
			if(enemyBullets[i]->move())
			{
				delete enemyBullets[i];
				enemyBullets.erase(enemyBullets.begin()+i);
			}
		}

		//Add a skeleton from the top of the screen.
		--skeletonTimer;
		if(skeletonTimer == 0)
		{
			vector<int> startPoints;
			skeletonTimer = (int)(Random::getGlobal().getRandomFloat() * 30.0f) + 60.0f;

			if(player.getY() > 32.0f)
			{
				for(i=0;i<MapWidth;++i)
				{
					if(map[i] < 64)
						startPoints.push_back(i);
				}
				tempint = (int)(Random::getGlobal().getRandomFloat() * ((float)startPoints.size()-0.5f));
				skeletons.push_back(new Skeleton(startPoints[tempint] * 32.0f, -32.0f));
			}
		}


		//Move the skeletons.
		for(i=0;i<skeletons.size();++i)
		{
			skeletons[i]->move(map,
							   MapWidth,
							   MapHeight,
							   player.getX()+8.0f,
							   player.getY()+12.0f);
		}
		//Move the enemies.
		for(i=0;i<enemies.size();++i)
		{
			if(enemies[i]->move())
				enemyBullets.push_back(new ShipBullet(enemies[i]->getX()+10.0f, enemies[i]->getY()+24.0f, true));
		}

		//Move the boss.
		if(boss.move())
		{
			shake = 0.0f;

			tempint = (int)(boss.getY()+(32.0f*7.0f))/32;
			for(y=0;y<=tempint;++y)
			{
				if(y < MapHeight)
				{
					for(x=0;x<MapWidth;++x)
					{
						if(map[(y*MapWidth)+x] > 63)
						{
							map[(y*MapWidth)+x] = 0;
							explosions.push_back(new Explosion((x*32.0f), (y*32.0f)));

							Sounds::getInstance().playSound(Sounds::TileExplode);
						}
					}
				}
			}
		}

		//Do the bullet collisions.
		bulletCollisions();

		//Move the player.
		player.move(map, MapWidth, MapHeight);

		//Do the player-enemy collisions.
		playerCollisions();

		//Do the player-pickup collisions.
		pickupCollisions();

		//If the boss has gone off the bottom of the level and the player didn't
		//kill it, game over.
		if(boss.getY() > 320.0f)
			gameOver = true;
	}

	//Move the particles.
	for(i=0;i<NumParticles;++i)
		particles[i].move();

	//Draw the background tiles.
	x = 0;
	y = 0;
	for(i=0;i<(MapWidth*MapHeight);++i)
	{
		if(map[i] < 64)
			d->drawImage(tileSet->getGroundTile(map[i]), x+shakeOffset, y, tileScale, tileScale);
		else if(map[i] < 128)
			d->drawImage(tileSet->getWallTile(map[i]-64), x+shakeOffset, y, tileScale, tileScale);
		else if(map[i] < 192)
			d->drawImage(tileSet->getImpassableTile(map[i]-128), x+shakeOffset, y, tileScale, tileScale);
		else
			d->drawImage(tileSet->getGroundTile(0), x+shakeOffset, y, tileScale, tileScale);

		x += 32;
		if(x >= 224)
		{
			x = 0;
			y += 32;
		}
	}

	//Draw the pickups.
	for(i=0;i<pickups.size();++i)
		pickups[i]->draw(d);

	//Draw the skeletons.
	for(i=0;i<skeletons.size();++i)
		skeletons[i]->draw(d);

	//Draw the player's bullets.
	for(i=0;i<bullets.size();++i)
		bullets[i]->draw(d);

	//Draw the player.
	player.draw(d);

	//Draw the boss.
	boss.draw(d);

	//Draw the enemies' bullets.
	for(i=0;i<enemyBullets.size();++i)
		enemyBullets[i]->draw(d);

	//Draw the enemies.
	for(i=0;i<enemies.size();++i)
		enemies[i]->draw(d);

	//Draw the particles.
	for(i=0;i<NumParticles;++i)
		particles[i].draw(d);

	//Draw the explosions.
	for(i=(explosions.size()-1);i>=0;--i)
	{
		if(explosions[i]->draw(d))
		{
			delete explosions[i];
			explosions.erase(explosions.begin()+i);
		}
	}

	//Draw the score.
	d->drawText("score", 2, 2, 16.0f);
	{
		stringstream tempstr;

		tempstr << GameCounter::getInstance().getScore();
		d->drawText(tempstr.str(), 60, 2, 16.0f);
	}

	//Draw the player's lives.
	if(player.getLives() > 2)
		d->drawImage(heartImage, 152, 2, 2.0f, 2.0f);
	if(player.getLives() > 1)
		d->drawImage(heartImage, 176, 2, 2.0f, 2.0f);
	if(player.getLives() > 0)
		d->drawImage(heartImage, 200, 2, 2.0f, 2.0f);

	//Draw the number of big bullets the player has, if they have them.
	if(player.getNumBigBullets() > 0)
		d->fillRect(2, 18, player.getNumBigBullets(), 8, 255, 0, 0, 255);

	if(paused)
	{
		d->fillRect(0, 0, 224, 320, 0, 0, 0, 128);

		d->drawText("paused", 50, 40, 32.0f);

		d->drawText("esc to quit", 57, 200, 16.0f);
		d->drawText("fire to resume", 42, 218, 16.0f);
	}
	else if(gameOver)
	{
		d->drawText("game", 75, 80, 32.0f);
		d->drawText("over", 75, 112, 32.0f);
		d->drawText("press fire", 60, 300, 16.0f);
	}
	else if(levelComplete)
	{
		d->drawText("level", 60, 80, 32.0f);
		d->drawText("complete", 30, 112, 32.0f);

		if(pacifist)
			d->drawText("pacifist bonus", 40, 218, 16.0f, 0.5f, 0.5f, 1.0f);

		d->drawText("press fire", 60, 300, 16.0f);
	}
}

//------------------------------------------------------------------------------
void FinalGauntletGamePlay::keyDown(const SDL_keysym& keyPressed)
{
	if(keyPressed.sym == SDLK_ESCAPE)
	{
		if(!paused)
			paused = true;
		else
			nextState("TitleScreen");
	}
	else if(keyPressed.sym == SDLK_LEFT)
		player.moveLeft(true);
	else if(keyPressed.sym == SDLK_RIGHT)
		player.moveRight(true);
	else if(keyPressed.sym == SDLK_UP)
		player.moveUp(true);
	else if(keyPressed.sym == SDLK_DOWN)
		player.moveDown(true);
	else if(keyPressed.sym == SDLK_z)
	{
		if(paused)
			paused = false;
		else if(gameOver)
			nextState("FailScreen");
		else if(levelComplete)
			nextState("ShooterGamePlay");
		else
			fireIsDown = true;
	}
}

//------------------------------------------------------------------------------
void FinalGauntletGamePlay::keyUp(const SDL_keysym& keyPressed)
{
	if(keyPressed.sym == SDLK_LEFT)
		player.moveLeft(false);
	else if(keyPressed.sym == SDLK_RIGHT)
		player.moveRight(false);
	else if(keyPressed.sym == SDLK_UP)
		player.moveUp(false);
	else if(keyPressed.sym == SDLK_DOWN)
		player.moveDown(false);
	else if(keyPressed.sym == SDLK_z)
		fireIsDown = false;
}

//------------------------------------------------------------------------------
void FinalGauntletGamePlay::bulletCollisions()
{
	int x, y;
	int i, j, k;
	float dist;
	float xB, yB;
	float xS, yS;
	float xP, yP;
	int bossY = (int)boss.getY();
	bool hitEnemy = false;

	xP = player.getX() + 8.0f;
	yP = player.getY() + 12.0f;

	for(i=(bullets.size()-1);i>=0;--i)
	{
		x = (int)bullets[i]->getX()/32;
		y = (int)bullets[i]->getY()/32;

		if(bullets[i]->getBig())
			k = 4;
		else
			k = 1;

		//Test against boss.
		if((x >= 1) && (x < 6))
		{
			if(y < (bossY + (3.5f * 32.0f))/32.0f)
			{
				if(boss.decrementHealth(k))
				{
					explosions.push_back(new Explosion(0.0f, boss.getY(), true));

					Sounds::getInstance().playSound(Sounds::SkeletonDie);

					GameCounter::getInstance().incrementCount("Boss Killed", 1000000);

					levelComplete = true;
				}
				else
				{
					Sounds::getInstance().playSound(Sounds::BulletHit);

					for(k=0;k<NumParticles;++k)
					{
						if(!particles[k].isActive())
						{
							particles[k].setColour(1.0f, 0.0f, 0.0f);
							particles[k].setSize(32.0f, 32.0f);
							particles[k].setDensity(0.2f);
							particles[k].setDirection(0.1f, 1.0f);
							particles[k].setPosition(bullets[i]->getX(), bullets[i]->getY());
							particles[k].start(15);
							break;
						}
					}
				}

				delete bullets[i];
				bullets.erase(bullets.begin()+i);

				continue;
			}
			else if((x >= 2) && (x < 5) && y < (bossY + (6.5f * 32.0f))/32.0f)
			{
				if(boss.decrementHealth(k))
				{
					explosions.push_back(new Explosion(0.0f, boss.getY(), true));

					Sounds::getInstance().playSound(Sounds::SkeletonDie);

					GameCounter::getInstance().incrementCount("Enemy Killed", 500);

					levelComplete = true;
				}
				else
				{
					Sounds::getInstance().playSound(Sounds::BulletHit);

					for(k=0;k<NumParticles;++k)
					{
						if(!particles[k].isActive())
						{
							particles[k].setColour(1.0f, 0.0f, 0.0f);
							particles[k].setSize(32.0f, 32.0f);
							particles[k].setDensity(0.2f);
							particles[k].setDirection(0.1f, 1.0f);
							particles[k].setPosition(bullets[i]->getX(), bullets[i]->getY()-32.0f);
							particles[k].start(15);
							break;
						}
					}
				}

				delete bullets[i];
				bullets.erase(bullets.begin()+i);

				continue;
			}
		}

		//Test against walls.
		if((map[(y*MapWidth)+x] > 63) && (map[(y*MapWidth)+x] < 128))
		{
			//Add some particles.
			for(j=0;j<NumParticles;++j)
			{
				if(!particles[j].isActive())
				{
					particles[j].setColour(1.0f, 1.0f, 1.0f);
					particles[j].setSize(6.0f, 6.0f);
					particles[j].setDensity(0.1f);
					particles[j].setDirection(0.1f, -0.2f);
					particles[j].setPosition(bullets[i]->getX(), bullets[i]->getY());
					particles[j].start(15);
					break;
				}
			}

			--walls[(y*MapWidth)+x];
			if(bullets[i]->getBig())
			{
				if(walls[(y*MapWidth)+x] > 0)
					--walls[(y*MapWidth)+x];
				if(walls[(y*MapWidth)+x] > 0)
					--walls[(y*MapWidth)+x];
				if(walls[(y*MapWidth)+x] > 0)
					--walls[(y*MapWidth)+x];
			}

			if(walls[(y*MapWidth)+x] == 0)
			{
				map[(y*MapWidth)+x] = 0;
				explosions.push_back(new Explosion((x*32.0f), (y*32.0f)));

				Sounds::getInstance().playSound(Sounds::TileExplode);

				GameCounter::getInstance().incrementCount("Wall Destroyed", 100);
			}
			else
				Sounds::getInstance().playSound(Sounds::BulletHit);

			delete bullets[i];
			bullets.erase(bullets.begin()+i);
			break;
		}

		//Test against enemies.
		xB = bullets[i]->getX();
		yB = bullets[i]->getY();
		for(j=0;j<enemies.size();++j)
		{
			xS = enemies[j]->getX()+16.0f;
			yS = enemies[j]->getY()+16.0f;

			xS -= xB;
			yS -= yB;

			dist = sqrtf((xS*xS) + (yS*yS));
			if(dist < 16.0f)
			{
				//Add some particles.
				for(k=0;k<NumParticles;++k)
				{
					if(!particles[k].isActive())
					{
						particles[k].setColour(1.0f, 0.0f, 0.0f);
						particles[k].setSize(8.0f, 8.0f);
						particles[k].setDensity(0.2f);
						particles[k].setDirection(0.1f, 0.5f);
						particles[k].setPosition(bullets[i]->getX(), bullets[i]->getY());
						particles[k].start(15);
						break;
					}
				}

				if(bullets[i]->getBig())
					k = 4;
				else
					k = 1;
				if(enemies[j]->decrementLives(k))
				{
					explosions.push_back(new Explosion(enemies[j]->getX(), enemies[j]->getY()));
					delete enemies[j];
					enemies.erase(enemies.begin()+j);

					Sounds::getInstance().playSound(Sounds::SkeletonDie);

					GameCounter::getInstance().incrementCount("Enemy Killed", 500);
				}
				else
					Sounds::getInstance().playSound(Sounds::BulletHit);
				delete bullets[i];
				bullets.erase(bullets.begin()+i);

				hitEnemy = true;
				break;
			}
		}

		if(hitEnemy)
			continue;

		//Test against skeletons.
		xB = bullets[i]->getX();
		yB = bullets[i]->getY();
		for(j=0;j<skeletons.size();++j)
		{
			xS = skeletons[j]->getX()+16.0f;
			yS = skeletons[j]->getY()+16.0f;

			xS -= xB;
			yS -= yB;

			dist = sqrtf((xS*xS) + (yS*yS));
			if(dist < 12.0f)
			{
				//Add some particles.
				for(k=0;k<NumParticles;++k)
				{
					if(!particles[k].isActive())
					{
						particles[k].setColour(1.0f, 0.0f, 0.0f);
						particles[k].setSize(8.0f, 8.0f);
						particles[k].setDensity(0.2f);
						particles[k].setDirection(0.1f, 0.5f);
						particles[k].setPosition(bullets[i]->getX(), bullets[i]->getY());
						particles[k].start(15);
						break;
					}
				}

				if(bullets[i]->getBig())
					k = 4;
				else
					k = 1;
				if(skeletons[j]->decrementHealth(k))
				{
					explosions.push_back(new Explosion(skeletons[j]->getX(), skeletons[j]->getY()));
					delete skeletons[j];
					skeletons.erase(skeletons.begin()+j);

					Sounds::getInstance().playSound(Sounds::SkeletonDie);

					GameCounter::getInstance().incrementCount("Skeleton Killed", 500);
				}
				else
					Sounds::getInstance().playSound(Sounds::BulletHit);
				delete bullets[i];
				bullets.erase(bullets.begin()+i);

				break;
			}
		}
	}

	//The enemy bullets.
	for(i=(enemyBullets.size()-1);i>=0;--i)
	{
		x = (int)enemyBullets[i]->getX()/32;
		y = (int)enemyBullets[i]->getY()/32;

		//Test against walls.
		if((map[(y*MapWidth)+x] > 63) && (map[(y*MapWidth)+x] < 128))
		{
			map[(y*MapWidth)+x] = 0;
			explosions.push_back(new Explosion((x*32.0f), (y*32.0f)));

			Sounds::getInstance().playSound(Sounds::TileExplode);

			delete enemyBullets[i];
			enemyBullets.erase(enemyBullets.begin()+i);
			break;
		}

		//Test against the player.
		xB = enemyBullets[i]->getX() + 8.0f;
		yB = enemyBullets[i]->getY() + 4.0f;
		xB -= (xP + 8.0f);
		yB -= (yP + 12.0f);

		dist = sqrtf((xB*xB) + (yB*yB));
		if(dist < 8.0f)
		{
			//Get rid of the bullet.
			delete enemyBullets[i];
			enemyBullets.erase(enemyBullets.begin()+i);

			Sounds::getInstance().playSound(Sounds::GauntletDie);

			explosions.push_back(new Explosion(player.getX(), player.getY()));

			if(player.decrementLives())
				gameOver = true;
			else
				Sounds::getInstance().playSound(Sounds::GauntletNew);
		}
	}
}

//------------------------------------------------------------------------------
void FinalGauntletGamePlay::playerCollisions()
{
	int i;
	float dist;
	float xP, yP;
	float xS, yS;

	xP = player.getX() + 8.0f;
	yP = player.getY() + 12.0f;
	//Skeletons.
	for(i=0;i<skeletons.size();++i)
	{
		xS = skeletons[i]->getX()+16.0f;
		yS = skeletons[i]->getY()+16.0f;

		xS -= xP;
		yS -= yP;

		dist = sqrtf((xS*xS) + (yS*yS));
		if(dist < 8.0f)
		{
			explosions.push_back(new Explosion(skeletons[i]->getX(), skeletons[i]->getY()));

			delete skeletons[i];
			skeletons.erase(skeletons.begin()+i);

			Sounds::getInstance().playSound(Sounds::GauntletDie);

			if(player.decrementLives())
				gameOver = true;
			else
				Sounds::getInstance().playSound(Sounds::GauntletNew);

			return;
		}
	}
	//Enemies.
	for(i=0;i<enemies.size();++i)
	{
		xS = enemies[i]->getX()+16.0f;
		yS = enemies[i]->getY()+16.0f;

		xS -= xP;
		yS -= yP;

		dist = sqrtf((xS*xS) + (yS*yS));
		if(dist < 8.0f)
		{
			explosions.push_back(new Explosion(enemies[i]->getX(), enemies[i]->getY()));

			delete enemies[i];
			enemies.erase(enemies.begin()+i);

			Sounds::getInstance().playSound(Sounds::GauntletDie);

			if(player.decrementLives())
				gameOver = true;
			else
				Sounds::getInstance().playSound(Sounds::GauntletNew);

			return;
		}
	}
	//Boss.
	yP -= 12.0f;
	if((xP > 32.0f) && (xP < (6.0f*32.0f)))
	{
		if(yP < (boss.getY() + (3.5f * 32.0f)))
		{
			Sounds::getInstance().playSound(Sounds::GauntletDie);

			if(player.decrementLives())
				gameOver = true;
			else
				Sounds::getInstance().playSound(Sounds::GauntletNew);
		}
	}
	else if((xP > 64.0f) && (xP < (5.0f*32.0f)))
	{
		if(yP < (boss.getY() + (6.5f * 32.0f)))
		{
			Sounds::getInstance().playSound(Sounds::GauntletDie);

			if(player.decrementLives())
				gameOver = true;
			else
				Sounds::getInstance().playSound(Sounds::GauntletNew);
		}
	}
}

//------------------------------------------------------------------------------
void FinalGauntletGamePlay::pickupCollisions()
{
	int i;
	float dist;
	float xP, yP;
	float x2, y2;

	xP = player.getX() + 8.0f;
	yP = player.getY() + 12.0f;

	for(i=(pickups.size()-1);i>=0;--i)
	{
		x2 = pickups[i]->getX()+16.0f;
		y2 = pickups[i]->getY()+16.0f;

		x2 -= xP;
		y2 -= yP;

		dist = sqrtf((x2*x2) + (y2*y2));
		if(dist < 10.0f)
		{
			pickups[i]->apply(&player);
			delete pickups[i];
			pickups.erase(pickups.begin()+i);

			Sounds::getInstance().playSound(Sounds::Pickup);

			GameCounter::getInstance().incrementCount("Picked up Pickup", 100);
		}
	}
}
