//	GauntletGamePlay.cpp - The gauntlet state.
//  ----------------------------------------------------------------------------
//	This file is part of 'Tile Massacre Shmup', my entry to the TIGSource
//	Assemblee contest.
//	Copyright (C) 2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifdef DEBUG
#pragma warning(disable : 4786) //Stupid MS headers...
#endif

#include "GauntletGamePlay.h"
#include "NiallsDrawingLib/Drawer.h"
#include "NiallsDrawingLib/ImageLoader.h"
#include "Images.h"
#include "TileSet.h"
#include "Skeleton.h"
#include "Explosion.h"
#include "EnemySpawner.h"
#include "GauntletBullet.h"
#include "GameCounter.h"
#include "Random.h"
#include "HealthPickup.h"
#include "BulletPickup.h"
#include "sound/Sounds.h"
#include "TextScroller.h"
#include "DebugStatement.h"

#include <cmath>
#include <cassert>
#include <sstream>

using namespace std;

//------------------------------------------------------------------------------
uint32_t GauntletGamePlay::heartImage = 0;
uint32_t GauntletGamePlay::exitArrow = 0;
uint32_t GauntletGamePlay::particle1 = 0;
uint32_t GauntletGamePlay::particle2 = 0;

//------------------------------------------------------------------------------
GauntletGamePlay::GauntletGamePlay(Application *app, uint8_t *level, TileSet *tiles, int index):
GameState(app),
tileSet(tiles),
tutorial(0),
arrowBounce(0.0f),
fireIsDown(false),
bulletTimer(0),
pacifist(true),
gameOver(false),
levelComplete(false),
paused(false)
{
	int i;
	int x, y;
	int tempint;
	int numGround, numWall, numImpassable;

	if(index == 1)
		tutorial = new TextScroller("get to the exit at the top of the screen");

	//Initialise Skeleton timer.
	skeletonTimer = (int)(Random::getGlobal().getRandomFloat() * 60.0f) + 300.0f;

	numGround = tileSet->getNumGroundTiles();
	numWall = tileSet->getNumWallTiles();
	numImpassable = tileSet->getNumImpassableTiles();

	//Copy the map across.
	for(i=0;i<(MapWidth*MapHeight);++i)
	{
		map[i] = level[i];
		walls[i] = 32;

		
		if(map[i] < 63)
		{
			assert(map[i] < numGround);
			if(map[i] >= numGround)
			{
				DebugStatement("Ground tile out of range:");
				DebugStatement(map[i]);
				map[i] = 0;
			}
		}
		else if(map[i] < 128)
		{
			assert((map[i]-64) < numWall);
			if((map[i]-64) >= numWall)
			{
				DebugStatement("Wall tile out of range:");
				DebugStatement(map[i]);
				map[i] = 0;
			}
		}
		else if(map[i] < 192)
		{
			assert((map[i]-128) < numImpassable);
			if((map[i]-128) >= numImpassable)
			{
				DebugStatement("Impassable tile out of range:");
				DebugStatement(map[i]);
				map[i] = 0;
			}
		}
		else
		{
			assert(map[i] == 192);
			if(map[i] > 192)
			{
				DebugStatement("Spawner tile out of range:");
				DebugStatement(map[i]);
				map[i] = 0;
			}
		}
	}

	//Make sure the entrance and exit to the level are clear.
	if(map[3] > 63)
		map[3] = 0;
	if(map[(9*MapWidth)+3] > 63)
		map[(9*MapWidth)+3] = 0;

	//Make sure the player hasn't blocked their way to the exit with impassable
	//tiles.
	for(y=0;y<(MapHeight-1);++y)
	{
		int impassCount = 0;
		int twoImpassCount = 0;

		for(x=0;x<MapWidth;++x)
		{
			//Check if the whole row is impassable.
			if((map[(y*MapWidth)+x] > 127) && (map[(y*MapWidth)+x] < 192))
				++impassCount;

			//Check if this row in conjunction with the next row is impassable.
			if((((map[(y*MapWidth)+x] < 128) || (map[(y*MapWidth)+x] > 191)) &&
			    ((map[((y+1)*MapWidth)+x] > 127) && (map[((y+1)*MapWidth)+x] < 192))) ||
			   (((map[(y*MapWidth)+x] > 127) && (map[(y*MapWidth)+x] < 192)) &&
			    ((map[((y+1)*MapWidth)+x] < 128) || (map[((y+1)*MapWidth)+x] > 191))) ||
			   (((map[(y*MapWidth)+x] > 127) && (map[(y*MapWidth)+x] < 192)) &&
			    ((map[((y+1)*MapWidth)+x] > 127) && (map[((y+1)*MapWidth)+x] < 192))))
			{
				++twoImpassCount;
			}
		}
		if(impassCount > (MapWidth-2))
		{
			tempint = -1;
			for(x=0;x<MapWidth;++x)
			{
				if((map[((y+1)*MapWidth)+x] > 127) && (map[(y*MapWidth)+x] < 192))
				{
					tempint = x;
					break;
				}
			}
			if(tempint > -1)
				map[(y*MapWidth)+x] = 0;
			else
			{
				tempint = (int)(Random::getGlobal().getRandomFloat() * (float)(MapWidth-1));
				map[(y*MapWidth)+tempint] = 0;
			}
		}

		if(twoImpassCount > (MapWidth-2))
		{
			//Make sure there's a path from one row to the next.
			tempint = (int)(Random::getGlobal().getRandomFloat() * (float)(MapWidth-1));
			for(x=(y+1);x>=0;--x)
				map[(x*MapWidth)+tempint] = 0;
		}
	}

	//Add any spawners to the map.
	for(i=0;i<(MapWidth*MapHeight);++i)
	{
		if(map[i] > 191)
		{
			x = i%MapWidth;
			y = i/MapWidth;
			spawners.push_back(new EnemySpawner(x*32, y*32, (y*MapWidth)+x));
		}
	}

	//Add some health pickups to the level.
	tempint = 3 + ((int)(Random::getGlobal().getRandomFloat() * 2.0f)-1);
	while(tempint > 0)
	{
		int failsafe = 0;

		i = (int)(Random::getGlobal().getRandomFloat() * (float)((MapWidth*MapHeight)-1));
		while(!((map[i] > 63) && (map[i] < 128)))
		{
			++i;
			i %= (MapWidth*MapHeight);

			++failsafe;
			if(failsafe >= (MapWidth*MapHeight))
				break;
		}
		if(failsafe >= (MapWidth*MapHeight))
			break;

		x = i%MapWidth;
		y = i/MapWidth;
		pickups.push_back(new HealthPickup((x*32), (y*32)));

		--tempint;
	}

	//Add some bullet pickups to the level.
	tempint = 3 + ((int)(Random::getGlobal().getRandomFloat() * 2.0f)-1);
	while(tempint > 0)
	{
		int failsafe = 0;

		i = (int)(Random::getGlobal().getRandomFloat() * (float)((MapWidth*MapHeight)-1));
		while(!((map[i] > 63) && (map[i] < 128)))
		{
			++i;
			i %= (MapWidth*MapHeight);

			++failsafe;
			if(failsafe >= (MapWidth*MapHeight))
				break;
		}
		if(failsafe >= (MapWidth*MapHeight))
			break;

		x = i%MapWidth;
		y = i/MapWidth;
		pickups.push_back(new BulletPickup((x*32), (y*32)));

		--tempint;
	}

	//Setup the particles.
	for(i=0;i<NumParticles;++i)
	{
		particles[i].setDensity(0.1f);
		particles[i].setDirection(0.1f, -0.2f);
		particles[i].setLifespan(3.0f);
		particles[i].setSize(6.0f, 6.0f);
		particles[i].setRandomSize(true);
		if(particle1 != 0)
		{
			particles[i].addImage(particle1);
			particles[i].addImage(particle2);
		}
	}
}

//------------------------------------------------------------------------------
GauntletGamePlay::~GauntletGamePlay()
{
	int i;

	for(i=0;i<spawners.size();++i)
		delete spawners[i];
	for(i=0;i<bullets.size();++i)
		delete bullets[i];
	for(i=0;i<explosions.size();++i)
		delete explosions[i];
	for(i=0;i<skeletons.size();++i)
		delete skeletons[i];
	for(i=0;i<pickups.size();++i)
		delete pickups[i];

	if(tutorial)
		delete tutorial;
}

//------------------------------------------------------------------------------
void GauntletGamePlay::draw(Drawer *d)
{
	int i;
	int x, y;
	int tempint;
	float tileScale = 2.0f;

	if(!exitArrow)
	{
		ImageLoader imLoad;
		ImageData data;

		data = imLoad.loadImage(Images::exitarrow_png, Images::exitarrow_pngSize);
		exitArrow = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::heart_png, Images::heart_pngSize);
		heartImage = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::particle1_png, Images::particle1_pngSize);
		particle1 = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::particle2_png, Images::particle2_pngSize);
		particle2 = d->loadImage(data.data, data.width, data.height);

		for(i=0;i<NumParticles;++i)
		{
			particles[i].addImage(particle1);
			particles[i].addImage(particle2);
		}
	}

	//Set the tile scale factor to account for tiles that aren't 16x16.
	if(tileSet->getTileSize() != 16)
		tileScale = 32.0f/(float)tileSet->getTileSize();

	if(!gameOver && !levelComplete && !paused)
	{
		//Fire a new bullet if necessary.
		if(fireIsDown && !bulletTimer)
		{
			bool big = (player.getNumBigBullets() > 0);
			float xDir, yDir;

			bulletTimer = 5;

			if(player.getLeft())
				xDir = -1.0f;
			else if(player.getRight())
				xDir = 1.0f;
			else
				xDir = 0.0f;
			if(player.getUp())
				yDir = -1.0f;
			else if(player.getDown())
				yDir = 1.0f;
			else
				yDir = 0.0f;
			if((xDir == 0.0f) && (yDir == 0.0f))
			{
				switch(player.getImage())
				{
					case 0:
						xDir = 0.0f;
						yDir = -1.0f;
						break;
					case 1:
						xDir = 1.0f;
						yDir = 0.0f;
						break;
					case 2:
						xDir = 0.0f;
						yDir = 1.0f;
						break;
					case 3:
						xDir = -1.0f;
						yDir = 0.0f;
						break;
				}
			}
			bullets.push_back(new GauntletBullet(player.getX()+8.0f,
												 player.getY()+8.0f,
												 xDir,
												 yDir,
												 big));
			if(big)
				player.decrementBigBullets();
			pacifist = false;

			Sounds::getInstance().playSound(Sounds::GauntletGun);
		}
		else if(bulletTimer > 0)
			--bulletTimer;

		//Move the bullets.
		for(i=(bullets.size()-1);i>=0;--i)
		{
			if(bullets[i]->move())
			{
				delete bullets[i];
				bullets.erase(bullets.begin()+i);
			}
		}

		//Add a skeleton from the top of the screen.
		--skeletonTimer;
		if(skeletonTimer == 0)
		{
			vector<int> startPoints;
			skeletonTimer = (int)(Random::getGlobal().getRandomFloat() * 60.0f) + 300.0f;

			if(player.getY() > 32.0f)
			{
				for(i=0;i<MapWidth;++i)
				{
					if(map[i] < 64)
						startPoints.push_back(i);
				}
				tempint = (int)(Random::getGlobal().getRandomFloat() * ((float)startPoints.size()-0.5f));
				skeletons.push_back(new Skeleton(startPoints[tempint] * 32.0f, -32.0f));
			}
		}

		//Check whether it's time to spawn a new enemy.
		for(i=0;i<spawners.size();++i)
		{
			if(spawners[i]->spawnTime())
			{
				//Spawn a new enemy.
				skeletons.push_back(new Skeleton(spawners[i]->getX(), spawners[i]->getY()));
			}
		}

		//Move the skeletons.
		for(i=0;i<skeletons.size();++i)
		{
			skeletons[i]->move(map,
							   MapWidth,
							   MapHeight,
							   player.getX()+8.0f,
							   player.getY()+12.0f);
		}

		//Do the bullet collisions.
		bulletCollisions();

		//Move the player.
		player.move(map, MapWidth, MapHeight);

		//Do the player-skeleton collisions.
		playerCollisions();

		//Do the player-pickup collisions.
		pickupCollisions();

		//If the player's off the top of the level, that's the level completed.
		if(player.getY() <= -24.0f)
		{
			levelComplete = true;
			GameCounter::getInstance().incrementCount("Gauntlet Level Complete", 10000);

			if(spawners.size() == 0)
				GameCounter::getInstance().incrementCount("Destroyed all Spawners", 10000);
		}
	}

	//Move the particles.
	for(i=0;i<NumParticles;++i)
		particles[i].move();

	//Draw the background tiles.
	x = 0;
	y = 0;
	for(i=0;i<(MapWidth*MapHeight);++i)
	{
		if(map[i] < 64)
			d->drawImage(tileSet->getGroundTile(map[i]), x, y, tileScale, tileScale);
		else if(map[i] < 128)
			d->drawImage(tileSet->getWallTile(map[i]-64), x, y, tileScale, tileScale);
		else if(map[i] < 192)
			d->drawImage(tileSet->getImpassableTile(map[i]-128), x, y, tileScale, tileScale);
		else
			d->drawImage(tileSet->getGroundTile(0), x, y, tileScale, tileScale);

		x += 32;
		if(x >= 224)
		{
			x = 0;
			y += 32;
		}
	}

	//Draw the pickups.
	for(i=0;i<pickups.size();++i)
		pickups[i]->draw(d);

	//Draw the enemy spawners.
	for(i=0;i<spawners.size();++i)
		spawners[i]->draw(d);

	//Draw the skeletons.
	for(i=0;i<skeletons.size();++i)
		skeletons[i]->draw(d);

	//Draw the player's bullets.
	for(i=0;i<bullets.size();++i)
		bullets[i]->draw(d);

	//Draw the player.
	player.draw(d);

	//Draw the particles.
	for(i=0;i<NumParticles;++i)
		particles[i].draw(d);

	//Draw the explosions.
	for(i=(explosions.size()-1);i>=0;--i)
	{
		if(explosions[i]->draw(d))
		{
			delete explosions[i];
			explosions.erase(explosions.begin()+i);
		}
	}

	//Draw the exit arrow.
	arrowBounce += 0.05f;
	if(arrowBounce > 1.0f)
		arrowBounce = 0.0f;
	y = 10 + (int)(sinf(arrowBounce * 2.0f * 3.14159265f) * 4.0f);
	for(i=0;i<7;++i)
	{
		if(map[i] < 64)
			d->drawImage(exitArrow, (i*32.0f)+8.0f, y);
	}

	//Draw the score.
	d->drawText("score", 2, 2, 16.0f);
	{
		stringstream tempstr;

		tempstr << GameCounter::getInstance().getScore();
		d->drawText(tempstr.str(), 60, 2, 16.0f);
	}

	//Draw the player's lives.
	if(player.getLives() > 2)
		d->drawImage(heartImage, 152, 2, 2.0f, 2.0f);
	if(player.getLives() > 1)
		d->drawImage(heartImage, 176, 2, 2.0f, 2.0f);
	if(player.getLives() > 0)
		d->drawImage(heartImage, 200, 2, 2.0f, 2.0f);

	//Draw the number of big bullets the player has, if they have them.
	if(player.getNumBigBullets() > 0)
		d->fillRect(2, 18, player.getNumBigBullets(), 8, 255, 0, 0, 255);

	if(tutorial)
		tutorial->draw(d, 96.0f);

	if(paused)
	{
		d->fillRect(0, 0, 224, 320, 0, 0, 0, 128);

		d->drawText("paused", 50, 40, 32.0f);

		d->drawText("esc to quit", 57, 200, 16.0f);
		d->drawText("fire to resume", 42, 218, 16.0f);
	}
	else if(gameOver)
	{
		d->drawText("game", 75, 80, 32.0f);
		d->drawText("over", 75, 112, 32.0f);
		d->drawText("press fire", 60, 300, 16.0f);
	}
	else if(levelComplete)
	{
		d->drawText("level", 60, 80, 32.0f);
		d->drawText("complete", 30, 112, 32.0f);

		if(spawners.size() == 0)
			d->drawText("no spawner bonus", 30, 200, 16.0f, 1.0f, 0.0f, 0.0f);
		if(pacifist)
			d->drawText("pacifist bonus", 40, 218, 16.0f, 0.5f, 0.5f, 1.0f);

		d->drawText("press fire", 60, 300, 16.0f);
	}
}

//------------------------------------------------------------------------------
void GauntletGamePlay::keyDown(const SDL_keysym& keyPressed)
{
	if(keyPressed.sym == SDLK_ESCAPE)
	{
		if(!paused)
			paused = true;
		else
			nextState("TitleScreen");
	}
	else if(keyPressed.sym == SDLK_LEFT)
		player.moveLeft(true);
	else if(keyPressed.sym == SDLK_RIGHT)
		player.moveRight(true);
	else if(keyPressed.sym == SDLK_UP)
		player.moveUp(true);
	else if(keyPressed.sym == SDLK_DOWN)
		player.moveDown(true);
	else if(keyPressed.sym == SDLK_z)
	{
		if(paused)
			paused = false;
		else if(gameOver)
			nextState("FailScreen");
		else if(levelComplete)
			nextState("ShooterGamePlay");
		else
			fireIsDown = true;
	}
}

//------------------------------------------------------------------------------
void GauntletGamePlay::keyUp(const SDL_keysym& keyPressed)
{
	if(keyPressed.sym == SDLK_LEFT)
		player.moveLeft(false);
	else if(keyPressed.sym == SDLK_RIGHT)
		player.moveRight(false);
	else if(keyPressed.sym == SDLK_UP)
		player.moveUp(false);
	else if(keyPressed.sym == SDLK_DOWN)
		player.moveDown(false);
	else if(keyPressed.sym == SDLK_z)
		fireIsDown = false;
}

//------------------------------------------------------------------------------
void GauntletGamePlay::bulletCollisions()
{
	int x, y;
	int i, j, k;
	float dist;
	float xB, yB;
	float xS, yS;

	for(i=(bullets.size()-1);i>=0;--i)
	{
		x = (int)bullets[i]->getX()/32;
		y = (int)bullets[i]->getY()/32;

		//Test against walls.
		if((map[(y*MapWidth)+x] > 63) && (map[(y*MapWidth)+x] < 128))
		{
			//Add some particles.
			for(j=0;j<NumParticles;++j)
			{
				if(!particles[j].isActive())
				{
					particles[j].setColour(1.0f, 1.0f, 1.0f);
					particles[j].setSize(6.0f, 6.0f);
					particles[j].setDensity(0.1f);
					particles[j].setDirection(0.1f, -0.2f);
					particles[j].setPosition(bullets[i]->getX(), bullets[i]->getY());
					particles[j].start(15);
					break;
				}
			}

			--walls[(y*MapWidth)+x];
			if(bullets[i]->getBig())
			{
				if(walls[(y*MapWidth)+x] > 0)
					--walls[(y*MapWidth)+x];
				if(walls[(y*MapWidth)+x] > 0)
					--walls[(y*MapWidth)+x];
				if(walls[(y*MapWidth)+x] > 0)
					--walls[(y*MapWidth)+x];
			}

			if(walls[(y*MapWidth)+x] == 0)
			{
				map[(y*MapWidth)+x] = 0;
				explosions.push_back(new Explosion((x*32.0f), (y*32.0f)));

				Sounds::getInstance().playSound(Sounds::TileExplode);

				GameCounter::getInstance().incrementCount("Wall Destroyed", 100);
			}
			else
				Sounds::getInstance().playSound(Sounds::BulletHit);

			delete bullets[i];
			bullets.erase(bullets.begin()+i);
			break;
		}
		//Test against enemy spawners.
		else if(map[(y*MapWidth)+x] == 192)
		{
			bool hitSpawner = false;
			int tempint = (y*MapWidth)+x;

			for(j=0;j<spawners.size();++j)
			{
				if(spawners[j]->getIndex() == tempint)
				{
					int numHits = 1;

					hitSpawner = true;

					if(bullets[i]->getBig())
						numHits = 4;
					for(k=0;k<numHits;++k)
					{
						if(spawners[j]->decrementHealth())
						{
							map[(y*MapWidth)+x] = 0;

							explosions.push_back(new Explosion((x*32.0f), (y*32.0f)));
							delete spawners[j];
							spawners.erase(spawners.begin()+j);

							Sounds::getInstance().playSound(Sounds::TileExplode);

							GameCounter::getInstance().incrementCount("Destroyed Spawner", 1000);
							break;
						}
						else
							Sounds::getInstance().playSound(Sounds::BulletHit);
					}

					break;
				}
			}
			if(hitSpawner)
			{
				//Add some particles.
				for(k=0;k<NumParticles;++k)
				{
					if(!particles[k].isActive())
					{
						particles[k].setColour(1.0f, 1.0f, 1.0f);
						particles[k].setSize(6.0f, 6.0f);
						particles[k].setDensity(0.1f);
						particles[k].setDirection(0.1f, -0.2f);
						particles[k].setPosition(bullets[i]->getX(), bullets[i]->getY());
						particles[k].start(15);
						break;
					}
				}

				delete bullets[i];
				bullets.erase(bullets.begin()+i);
			}
		}

		//Test against skeletons.
		xB = bullets[i]->getX();
		yB = bullets[i]->getY();
		for(j=0;j<skeletons.size();++j)
		{
			xS = skeletons[j]->getX()+16.0f;
			yS = skeletons[j]->getY()+16.0f;

			xS -= xB;
			yS -= yB;

			dist = sqrtf((xS*xS) + (yS*yS));
			if(dist < 12.0f)
			{
				//Add some particles.
				for(k=0;k<NumParticles;++k)
				{
					if(!particles[k].isActive())
					{
						particles[k].setColour(1.0f, 0.0f, 0.0f);
						particles[k].setSize(8.0f, 8.0f);
						particles[k].setDensity(0.2f);
						particles[k].setDirection(0.1f, 0.5f);
						particles[k].setPosition(bullets[i]->getX(), bullets[i]->getY());
						particles[k].start(15);
						break;
					}
				}

				if(bullets[i]->getBig())
					k = 4;
				else
					k = 1;
				if(skeletons[j]->decrementHealth(k))
				{
					explosions.push_back(new Explosion(skeletons[j]->getX(), skeletons[j]->getY()));
					delete skeletons[j];
					skeletons.erase(skeletons.begin()+j);

					Sounds::getInstance().playSound(Sounds::SkeletonDie);

					GameCounter::getInstance().incrementCount("Skeleton Killed", 500);
				}
				else
					Sounds::getInstance().playSound(Sounds::BulletHit);
				delete bullets[i];
				bullets.erase(bullets.begin()+i);

				break;
			}
		}
	}
}

//------------------------------------------------------------------------------
void GauntletGamePlay::playerCollisions()
{
	int i;
	float dist;
	float xP, yP;
	float xS, yS;

	xP = player.getX() + 8.0f;
	yP = player.getY() + 12.0f;
	for(i=0;i<skeletons.size();++i)
	{
		xS = skeletons[i]->getX()+16.0f;
		yS = skeletons[i]->getY()+16.0f;

		xS -= xP;
		yS -= yP;

		dist = sqrtf((xS*xS) + (yS*yS));
		if(dist < 8.0f)
		{
			explosions.push_back(new Explosion(skeletons[i]->getX(), skeletons[i]->getY()));

			delete skeletons[i];
			skeletons.erase(skeletons.begin()+i);

			Sounds::getInstance().playSound(Sounds::GauntletDie);

			if(player.decrementLives())
				gameOver = true;
			else
				Sounds::getInstance().playSound(Sounds::GauntletNew);

			break;
		}
	}
}

//------------------------------------------------------------------------------
void GauntletGamePlay::pickupCollisions()
{
	int i;
	float dist;
	float xP, yP;
	float x2, y2;

	xP = player.getX() + 8.0f;
	yP = player.getY() + 12.0f;

	for(i=(pickups.size()-1);i>=0;--i)
	{
		x2 = pickups[i]->getX()+16.0f;
		y2 = pickups[i]->getY()+16.0f;

		x2 -= xP;
		y2 -= yP;

		dist = sqrtf((x2*x2) + (y2*y2));
		if(dist < 10.0f)
		{
			pickups[i]->apply(&player);
			delete pickups[i];
			pickups.erase(pickups.begin()+i);

			Sounds::getInstance().playSound(Sounds::Pickup);

			GameCounter::getInstance().incrementCount("Picked up Pickup", 100);
		}
	}
}
