//	LevelEditor.cpp - The level editor state.
//  ----------------------------------------------------------------------------
//	This file is part of 'Tile Massacre Shmup', my entry to the TIGSource
//	Assemblee contest.
//	Copyright (C) 2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "LevelEditor.h"
#include "NiallsDrawingLib/Drawer.h"
#include "NiallsDrawingLib/ImageLoader.h"
#include "Images.h"
#include "TileSet.h"
#include "TileSetFactory.h"
#include "FilePath.h"

#include <fstream>
#include <iostream>

using namespace std;

//------------------------------------------------------------------------------
LevelEditor::LevelEditor(Application *app):
GameState(app),
mouse(0),
tileSet(0),
xPos(0),
yPos(0),
dragging(false),
currentTile(0),
selectTileSet(true),
selectTile(false),
saveMode(false)
{
	int i;

	for(i=0;i<(MapWidth*MapHeight);++i)
		map[i] = 0;
}

//------------------------------------------------------------------------------
LevelEditor::~LevelEditor()
{
	
}

//------------------------------------------------------------------------------
void LevelEditor::draw(Drawer *d)
{
	int i;
	int x, y;
	float tileScale = 2.0f;

	if(!mouse)
	{
		ImageLoader imLoad;
		ImageData data;

		data = imLoad.loadImage(Images::mouse_png, Images::mouse_pngSize);
		mouse = d->loadImage(data.data, data.width, data.height);

		TileSetFactory::getInstance().loadTiles(d);
	}

	//Set the tile scale factor to account for tiles that aren't 16x16.
	if(tileSet)
	{
		if(tileSet->getTileSize() != 16)
			tileScale = 32.0f/(float)tileSet->getTileSize();
	}

	d->fillRect(0, 0, 224, 320, 0, 0, 0, 255);

	//If the user's selecting a tile set to use, draw the first tile from each
	//set.
	if(selectTileSet)
	{
		x = 0;
		y = 0;
		for(i=0;i<TileSetFactory::getInstance().getNumTileSets();++i)
		{
			d->drawImage(TileSetFactory::getInstance().getTileSet(i)->getGroundTile(0),
						 x,
						 y);
			x += 16;
			if(x >= 224)
			{
				x = 0;
				y += 16;
			}
		}
	}
	//If the user's selecting a particular tile to use.
	else if(selectTile)
	{
		x = 0;
		y = 0;

		//Ground tiles.
		for(i=0;i<tileSet->getNumGroundTiles();++i)
		{
			d->drawImage(tileSet->getGroundTile(i), x, y);
			x += 16;
			if(x >= 224)
			{
				x = 0;
				y += 16;
			}
		}

		//Wall tiles.
		for(i=0;i<tileSet->getNumWallTiles();++i)
		{
			d->drawImage(tileSet->getWallTile(i), x, y);
			x += 16;
			if(x >= 224)
			{
				x = 0;
				y += 16;
			}
		}

		//Impassable tiles.
		for(i=0;i<tileSet->getNumImpassableTiles();++i)
		{
			d->drawImage(tileSet->getImpassableTile(i), x, y);
			x += 16;
			if(x >= 224)
			{
				x = 0;
				y += 16;
			}
		}

		//Enemy Spawner.
		d->fillRect(x, y, 16, 16, 255, 0, 0, 255);
	}
	//Otherwise draw the map.
	else
	{
		x = 0;
		y = 0;
		for(i=0;i<(MapWidth*MapHeight);++i)
		{
			if(map[i] < 64)
				d->drawImage(tileSet->getGroundTile(map[i]), x, y, tileScale, tileScale);
			else if(map[i] < 128)
				d->drawImage(tileSet->getWallTile(map[i]-64), x, y, tileScale, tileScale);
			else if(map[i] < 192)
				d->drawImage(tileSet->getImpassableTile(map[i]-128), x, y, tileScale, tileScale);
			else
				d->fillRect(x, y, 32, 32, 255, 0, 0, 255);

			x += 32;
			if(x >= 224)
			{
				x = 0;
				y += 32;
			}
		}
	}

	d->drawImage(mouse, xPos-2, yPos-2);

	//Draw the save text.
	if(saveMode)
	{
		d->drawText("enter a level name", 1, 1, 10.0f);
		d->drawText(saveName.str(), 1, 14, 10.0f);
	}
}

//------------------------------------------------------------------------------
void LevelEditor::keyDown(const SDL_keysym& keyPressed)
{
	if(keyPressed.sym == SDLK_ESCAPE)
	{
		if(saveMode)
		{
			saveMode = false;
			saveName.str("");
			saveName << prevSaveName;
		}
		else
			quit();
	}
	else if((keyPressed.sym == SDLK_SPACE) && !selectTileSet)
		selectTile = true;
	else if(keyPressed.sym == SDLK_RIGHT)
	{
		++currentTile;
		if((currentTile < 64) && (currentTile >= tileSet->getNumGroundTiles()))
			currentTile = 64;
		else if((currentTile < 128) &&
				((currentTile-64) >= tileSet->getNumWallTiles()))
		{
			if(tileSet->getNumImpassableTiles() > 0)
				currentTile = 128;
			else
				currentTile = 0;
		}
		else if((currentTile < 192) &&
				((currentTile-128) >= tileSet->getNumImpassableTiles()))
		{
			currentTile = 0;
		}
	}
	else if(keyPressed.sym == SDLK_LEFT)
	{
		--currentTile;
		if(currentTile < 0)
		{
			if(tileSet->getNumImpassableTiles() > 0)
				currentTile = 128 + tileSet->getNumImpassableTiles() - 1;
			else
				currentTile = 64 + tileSet->getNumWallTiles() - 1;
		}
		else if((currentTile < 64) &&
				(currentTile >= tileSet->getNumGroundTiles()))
		{
			currentTile = tileSet->getNumGroundTiles() - 1;
		}
		else if((currentTile < 128) &&
		   ((currentTile-64) >= tileSet->getNumWallTiles()))
		{
			currentTile = 64 + tileSet->getNumWallTiles() - 1;
		}
	}
	else if(saveMode)
	{
		if(keyPressed.sym == SDLK_RETURN)
		{
			save();
			saveMode = false;
		}
		else if(keyPressed.sym == SDLK_BACKSPACE)
		{
			string tempstr = saveName.str().substr(0, saveName.str().length()-1);

			saveName.str(tempstr);
		}
		else if(isValidChar(keyPressed.sym))
			saveName << getChar(keyPressed.sym);
	}
	else if((keyPressed.sym == SDLK_s) && (keyPressed.mod == KMOD_LCTRL))
	{
		saveMode = true;
		prevSaveName = saveName.str();
	}
}

//------------------------------------------------------------------------------
void LevelEditor::mouseDown(int button, int x, int y)
{
	int index;

	//If the user's still selecting a tile set, click on the tile to choose it.
	if(selectTileSet)
	{
		x /= 16;
		y /= 16;

		index = (y*14) + x;

		if(index < TileSetFactory::getInstance().getNumTileSets())
		{
			tileSet = TileSetFactory::getInstance().getTileSet(index);
			selectTileSet = false;
		}
	}
	//If the user's selecting a tile to draw with.
	else if(selectTile)
	{
		x /= 16;
		y /= 16;

		index = (y*14) + x;

		if(index < tileSet->getNumGroundTiles())
		{
			currentTile = index;
			selectTile = false;
		}
		else if((index - tileSet->getNumGroundTiles()) < tileSet->getNumWallTiles())
		{
			currentTile = (index - tileSet->getNumGroundTiles()) + 64;
			selectTile = false;
		}
		else if((index - (tileSet->getNumGroundTiles() + tileSet->getNumWallTiles())) < tileSet->getNumImpassableTiles())
		{
			currentTile = (index - (tileSet->getNumGroundTiles() + tileSet->getNumWallTiles())) + 128;
			selectTile = false;
		}
		else
		{
			currentTile = 192;
			selectTile = false;
		}
	}
	//Otherwise draw with that tile.
	else
	{
		x /= 32;
		y /= 32;

		map[(y*MapWidth) + x] = currentTile;

		dragging = true;
	}

}

//------------------------------------------------------------------------------
void LevelEditor::mouseUp(int button, int x, int y)
{
	dragging = false;
}

//------------------------------------------------------------------------------
void LevelEditor::mouseMove(int x, int y)
{
	xPos = x;
	yPos = y;

	if(dragging)
	{
		x /= 32;
		y /= 32;

		map[(y*MapWidth) + x] = currentTile;
	}
}

//------------------------------------------------------------------------------
void LevelEditor::load(const string& path)
{
	int i;
	FilePath levelsPath = FilePath::getDataFilesDir().getChild("levels");

	if(!levelsPath.exists())
		levelsPath.createAsDirectory();

	FilePath filepath = levelsPath.getChild(path);

	//Load the file.
	{
		char buf;
		ifstream phil(filepath.getPath().c_str(), ios::in|ios::binary);

		if(phil.good())
		{
			phil.read(&buf, 1);
			tileSet = TileSetFactory::getInstance().getTileSet((uint8_t)buf);

			for(i=0;i<(MapWidth*MapHeight);++i)
			{
				phil.read(&buf, 1);
				map[i] = (uint8_t)buf;
			}

			selectTileSet = false;
		}
	}
}

//------------------------------------------------------------------------------
void LevelEditor::save()
{
	int i;
	int tileSetIndex = -1;

	//Create the path, and the necessary directories.
	FilePath levelsPath = FilePath::getDataFilesDir().getChild("levels");

	if(!levelsPath.exists())
		levelsPath.createAsDirectory();

	FilePath path = levelsPath.getChild(saveName.str());

	//Save the file.
	{
		ofstream phil(path.getPath().c_str(), ios::out|ios::binary);

		if(phil.good())
		{
			//Get the tile set index.
			for(i=0;i<TileSetFactory::getInstance().getNumTileSets();++i)
			{
				if(TileSetFactory::getInstance().getTileSet(i) == tileSet)
				{
					tileSetIndex = i;
					break;
				}
			}

			if(tileSetIndex > -1)
			{
				phil.write((char *)&tileSetIndex, 1);

				for(i=0;i<(MapWidth*MapHeight);++i)
					phil.write((char *)&(map[i]), 1);
			}
			else
				cout << "Could not find TileSet." << endl;
		}
	}
}

//------------------------------------------------------------------------------
bool LevelEditor::isValidChar(SDLKey sym) const
{
	bool retval = false;

	switch(sym)
	{
		case SDLK_0:
		case SDLK_1:
		case SDLK_2:
		case SDLK_3:
		case SDLK_4:
		case SDLK_5:
		case SDLK_6:
		case SDLK_7:
		case SDLK_8:
		case SDLK_9:
		case SDLK_a:
		case SDLK_b:
		case SDLK_c:
		case SDLK_d:
		case SDLK_e:
		case SDLK_f:
		case SDLK_g:
		case SDLK_h:
		case SDLK_i:
		case SDLK_j:
		case SDLK_k:
		case SDLK_l:
		case SDLK_m:
		case SDLK_n:
		case SDLK_o:
		case SDLK_p:
		case SDLK_q:
		case SDLK_r:
		case SDLK_s:
		case SDLK_t:
		case SDLK_u:
		case SDLK_v:
		case SDLK_w:
		case SDLK_x:
		case SDLK_y:
		case SDLK_z:
		case SDLK_PERIOD:
			retval = true;
	}

	return retval;
}

//------------------------------------------------------------------------------
char LevelEditor::getChar(SDLKey sym) const
{
	char retval = '0';

	switch(sym)
	{
		case SDLK_0:
			retval = '0';
			break;
		case SDLK_1:
			retval = '1';
			break;
		case SDLK_2:
			retval = '2';
			break;
		case SDLK_3:
			retval = '3';
			break;
		case SDLK_4:
			retval = '4';
			break;
		case SDLK_5:
			retval = '5';
			break;
		case SDLK_6:
			retval = '6';
			break;
		case SDLK_7:
			retval = '7';
			break;
		case SDLK_8:
			retval = '8';
			break;
		case SDLK_9:
			retval = '9';
			break;
		case SDLK_a:
			retval = 'a';
			break;
		case SDLK_b:
			retval = 'b';
			break;
		case SDLK_c:
			retval = 'c';
			break;
		case SDLK_d:
			retval = 'd';
			break;
		case SDLK_e:
			retval = 'e';
			break;
		case SDLK_f:
			retval = 'f';
			break;
		case SDLK_g:
			retval = 'g';
			break;
		case SDLK_h:
			retval = 'h';
			break;
		case SDLK_i:
			retval = 'i';
			break;
		case SDLK_j:
			retval = 'j';
			break;
		case SDLK_k:
			retval = 'k';
			break;
		case SDLK_l:
			retval = 'l';
			break;
		case SDLK_m:
			retval = 'm';
			break;
		case SDLK_n:
			retval = 'n';
			break;
		case SDLK_o:
			retval = 'o';
			break;
		case SDLK_p:
			retval = 'p';
			break;
		case SDLK_q:
			retval = 'q';
			break;
		case SDLK_r:
			retval = 'r';
			break;
		case SDLK_s:
			retval = 's';
			break;
		case SDLK_t:
			retval = 't';
			break;
		case SDLK_u:
			retval = 'u';
			break;
		case SDLK_v:
			retval = 'v';
			break;
		case SDLK_w:
			retval = 'w';
			break;
		case SDLK_x:
			retval = 'x';
			break;
		case SDLK_y:
			retval = 'y';
			break;
		case SDLK_z:
			retval = 'z';
			break;
		case SDLK_PERIOD:
			retval = '.';
			break;
	}

	return retval;
}
