//	ShooterGamePlay.cpp - The shooter state.
//  ----------------------------------------------------------------------------
//	This file is part of 'Tile Massacre Shmup', my entry to the TIGSource
//	Assemblee contest.
//	Copyright (C) 2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifdef DEBUG
#pragma warning(disable : 4786) //Stupid MS headers...
#endif

#include "ShooterGamePlay.h"
#include "NiallsDrawingLib/Drawer.h"
#include "NiallsDrawingLib/ImageLoader.h"
#include "Images.h"
#include "ShipBullet.h"
#include "FilePath.h"
#include "TileSet.h"
#include "TileSetFactory.h"
#include "ShooterTile.h"
#include "Random.h"
#include "Explosion.h"
#include "GameCounter.h"
#include "TextScroller.h"
#include "sound/Sounds.h"
#include "ShooterEnemy.h"
#include "DebugStatement.h"

#include <cmath>
#include <sstream>
#include <fstream>
#include <cassert>

using namespace std;

//------------------------------------------------------------------------------
uint32_t ShooterGamePlay::heartImage = 0;
uint32_t ShooterGamePlay::enemySpawnerImage = 0;
uint32_t ShooterGamePlay::particle1 = 0;
uint32_t ShooterGamePlay::particle2 = 0;
bool ShooterGamePlay::loadTiles = false;

//------------------------------------------------------------------------------
ShooterGamePlay::ShooterGamePlay(Application *app, int index):
GameState(app),
tutorial(0),
fireIsDown(false),
levelComplete(false),
gameOver(false),
paused(false),
bulletRate(0),
currentTile((MapWidth*MapHeight)-1),
currentFlash(0.0f),
tileSet(0),
levelIndex(index),
tilesTimer(0.0f),
tilesSpeed(1.0f),
matchedTiles(0),
oneTileOnly(false),
startTimer(120),
authorTimer(240)
{
	int i;

	DebugStatement("ShooterGamePlay::ShooterGamePlay()");

	load();
	DebugStatement("ShooterGamePlay: level loaded");

	//Set any tutorial text.
	switch(index)
	{
		case 0:
			tutorial = new TextScroller("shoot the tiles to create a level    black tiles are ground    yellow tiles are walls    bonus points for matching the background");
			break;
		case 1:
			tutorial = new TextScroller("skulls spawn enemies");
			break;
		case 3:
			tutorial = new TextScroller("red tiles are impassable and cannot be destroyed    place them carefully");
			break;
	}

	while((map[currentTile] == 192) && (currentTile > -1))
		--currentTile;

	//Add enemies if necessary.
	if(index > 5)
	{
		int numEnemies = (int)((float)(index-5)*0.5f) + 1;
		for(i=0;i<numEnemies;++i)
			enemies.push_back(new ShooterEnemy());
		DebugStatement("ShooterGamePlay: Enemies added");
	}

	//Setup the particles.
	for(i=0;i<NumParticles;++i)
	{
		particles[i].setDensity(0.1f);
		particles[i].setDirection(0.1f, -0.2f);
		particles[i].setLifespan(3.0f);
		particles[i].setSize(6.0f, 6.0f);
		particles[i].setRandomSize(true);
		if(particle1 != 0)
		{
			particles[i].addImage(particle1);
			particles[i].addImage(particle2);
		}
	}
	DebugStatement("ShooterGamePlay: Particles setup.");
}

//------------------------------------------------------------------------------
ShooterGamePlay::~ShooterGamePlay()
{
	int i;

	for(i=0;i<bullets.size();++i)
		delete bullets[i];
	for(i=0;i<tiles.size();++i)
		delete tiles[i];
	for(i=0;i<explosions.size();++i)
		delete explosions[i];
	for(i=0;i<enemies.size();++i)
		delete enemies[i];

	if(tutorial)
		delete tutorial;
}

//------------------------------------------------------------------------------
void ShooterGamePlay::draw(Drawer *d)
{
	int i;
	int x, y;
	float tempf;
	float alpha;
	float tileScale = 2.0f;

	/*if(!loadTiles)
	{
		DebugStatement("ShooterGamePlay::draw()  Loading tiles.");
		TileSetFactory::getInstance().loadTiles(d);
		loadTiles = true;
		DebugStatement("ShooterGamePlay::draw()  Tiles loaded.");
	}*/

	if(!enemySpawnerImage)
	{
		ImageLoader imLoad;
		ImageData data;

		DebugStatement("ShooterGamePlay::draw()  Loading images.");

		data = imLoad.loadImage(Images::enemyspawner_png, Images::enemyspawner_pngSize);
		enemySpawnerImage = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::heart_png, Images::heart_pngSize);
		heartImage = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::particle1_png, Images::particle1_pngSize);
		particle1 = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::particle2_png, Images::particle2_pngSize);
		particle2 = d->loadImage(data.data, data.width, data.height);

		for(i=0;i<NumParticles;++i)
		{
			particles[i].addImage(particle1);
			particles[i].addImage(particle2);
		}

		DebugStatement("ShooterGamePlay::draw()  Images loaded.");
	}

	//Set the tile scale factor to account for tiles that aren't 16x16.
	if(tileSet->getTileSize() != 16)
		tileScale = 32.0f/(float)tileSet->getTileSize();

	if(!levelComplete && !gameOver && !paused && (startTimer == 0))
	{
		//Add a new (moving) tile if necessary.
		tilesTimer -= tilesSpeed;
		while(tilesTimer < 0.0f)
		{
			ShooterTile *tile = 0;

			//Add the next tile to shoot.
			if(Random::getGlobal().getRandomFloat() > 0.5f)
			{
				if(map[currentTile] < 64)
					tile = new ShooterTile(tileSet->getGroundTile(map[currentTile]), map[currentTile], tileScale);
				else if(map[currentTile] < 128)
					tile = new ShooterTile(tileSet->getWallTile(map[currentTile]-64), map[currentTile], tileScale);
				else
					tile = new ShooterTile(tileSet->getImpassableTile(map[currentTile]-128), map[currentTile], tileScale);
			}
			else if(Random::getGlobal().getRandomFloat() > 0.5f)
			{
				i = Random::getGlobal().getRandomFloat() * (tileSet->getNumGroundTiles()-0.001f);
				tile = new ShooterTile(tileSet->getGroundTile(i), i, tileScale);
			}
			else if(Random::getGlobal().getRandomFloat() > 0.5f)
			{
				i = Random::getGlobal().getRandomFloat() * (tileSet->getNumWallTiles()-0.001f);
				tile = new ShooterTile(tileSet->getWallTile(i), i+64, tileScale);
			}
			else if(tileSet->getNumImpassableTiles() > 0)
			{
				i = Random::getGlobal().getRandomFloat() * (tileSet->getNumImpassableTiles()-0.001f);
				tile = new ShooterTile(tileSet->getImpassableTile(i), i+128, tileScale);
			}
			else
			{
				i = Random::getGlobal().getRandomFloat() * (tileSet->getNumGroundTiles()-0.001f);
				tile = new ShooterTile(tileSet->getGroundTile(i), i, tileScale);
			}

			assert(tile != 0);
			tiles.push_back(tile);

			tilesTimer += (30.0f-((float)levelIndex*0.5f));
		}

		//Move any active tiles.
		for(i=(tiles.size()-1);i>=0;--i)
		{
			assert(tiles[i] != 0);
			if(tiles[i]->move())
			{
				delete tiles[i];
				tiles.erase(tiles.begin()+i);
			}
		}

		//Move any enemies.
		for(i=0;i<enemies.size();++i)
		{
			if(enemies[i]->move())
				bullets.push_back(new ShipBullet(enemies[i]->getX()+10.0f, enemies[i]->getY()+24.0f, true));
		}

		//Move any active bullets.
		for(i=(bullets.size()-1);i>=0;--i)
		{
			if(bullets[i]->move())
			{
				delete bullets[i];
				bullets.erase(bullets.begin()+i);
			}
		}

		//Add a new bullet if necessary.
		if(fireIsDown && !bulletRate)
		{
			bullets.push_back(new ShipBullet(player.getX()+2.0f, player.getY()-4));
			bulletRate = 10;

			Sounds::getInstance().playSound(Sounds::ShipGun);
		}
		else if(bulletRate > 0)
			--bulletRate;

		//Collision detection.
		collideTiles();
		collidePlayer();
	}
	else if(!paused)
	{
		//Move any active tiles.
		for(i=(tiles.size()-1);i>=0;--i)
		{
			assert(tiles[i] != 0);
			if(tiles[i]->move())
			{
				delete tiles[i];
				tiles.erase(tiles.begin()+i);
			}
		}
	}

	//Move the particles.
	for(i=0;i<NumParticles;++i)
		particles[i].move();

	//Draw the background tiles.
	x = 0;
	y = 0;
	if(startTimer > 30)
		alpha = 1.0f;
	else
		alpha = ((float)startTimer/60.0f) + 0.5f;
	for(i=0;i<(MapWidth*MapHeight);++i)
	{
		if(map[i] < 64)
			d->drawImage(tileSet->getGroundTile(map[i]), x, y, tileScale, tileScale, 1.0f, 1.0f, 1.0f, alpha);
		else if(map[i] < 128)
			d->drawImage(tileSet->getWallTile(map[i]-64), x, y, tileScale, tileScale, 1.0f, 1.0f, 1.0f, alpha);
		else if(map[i] < 192)
			d->drawImage(tileSet->getImpassableTile(map[i]-128), x, y, tileScale, tileScale, 1.0f, 1.0f, 1.0f, alpha);
		else
		{
			d->drawImage(tileSet->getGroundTile(0), x, y, tileScale, tileScale);
			d->drawImage(enemySpawnerImage, x, y, 1.0f, 1.0f, 1.0f);
		}

		x += 32;
		if(x >= 224)
		{
			x = 0;
			y += 32;
		}
	}

	//Draw the current tile the player's shooting for.
	if(startTimer == 0)
	{
		currentFlash += 0.0075f;
		if(currentFlash > 1.0f)
			currentFlash -= 1.0f;
		tempf = (sinf(currentFlash * 2.0f * 3.14159265f) + 1.0f)*0.25f;
		tempf += 0.25f;

		x = currentTile%MapWidth;
		y = currentTile/MapWidth;

		x *= 32;
		y *= 32;

		//d->drawImage(tileSet->getGroundTile(map[currentTile]), x, y, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, tempf);
		d->fillRect(x, y, 32, 32, 255, 255, 0, (uint8_t)(tempf*255));
	}

	//Draw the active tiles.
	if(!levelComplete)
	{
		for(i=0;i<tiles.size();++i)
		{
			if(!tiles[i]->isMoving())
				tiles[i]->draw(d);
		}
		for(i=0;i<tiles.size();++i)
		{
			if(tiles[i]->isMoving())
				tiles[i]->draw(d);
		}
	}
	else
	{
		for(i=0;(i<70)&&(i<tiles.size());++i)
			tiles[i]->draw(d);
	}

	//Draw any bullets the player's fired.
	for(i=0;i<bullets.size();++i)
		bullets[i]->draw(d);

	//Draw any enemies.
	if(!startTimer)
	{
		for(i=0;i<enemies.size();++i)
			enemies[i]->draw(d);
	}

	//Draw the player's ship.
	if(startTimer < 30)
		player.draw(d);

	//Draw the particles.
	for(i=0;i<NumParticles;++i)
		particles[i].draw(d);

	//Draw any explosions.
	for(i=(explosions.size()-1);i>=0;--i)
	{
		if(explosions[i]->draw(d))
		{
			delete explosions[i];
			explosions.erase(explosions.begin()+i);
		}
	}

	//Draw the score.
	d->drawText("score", 2, 2, 16.0f);
	{
		stringstream tempstr;

		tempstr << GameCounter::getInstance().getScore();
		d->drawText(tempstr.str(), 60, 2, 16.0f);
	}

	//Draw the player's lives.
	if(player.getLives() > 2)
		d->drawImage(heartImage, 152, 2, 2.0f, 2.0f);
	if(player.getLives() > 1)
		d->drawImage(heartImage, 176, 2, 2.0f, 2.0f);
	if(player.getLives() > 0)
		d->drawImage(heartImage, 200, 2, 2.0f, 2.0f);

	if(startTimer > 0)
	{
		--startTimer;

		alpha -= 0.5f;
		alpha *= 2.0f;
		d->drawText("create", 52, 40, 32.0f, 1.0f, 1.0f, 1.0f, alpha);
		d->drawText("this", 72, 65, 32.0f, 1.0f, 1.0f, 1.0f, alpha);
		d->drawText("level", 66, 90, 32.0f, 1.0f, 1.0f, 1.0f, alpha);
		d->drawText("for", 82, 115, 32.0f, 1.0f, 1.0f, 1.0f, alpha);
		d->drawText("bonus", 66, 140, 32.0f, 1.0f, 1.0f, 1.0f, alpha);
	}
	else if(tutorial)
		tutorial->draw(d, 96.0f);

	if(authorTimer > 0)
	{
		--authorTimer;

		if(authorTimer > 60)
			alpha = 1.0f;
		else
			alpha = (float)authorTimer/60.0f;

		d->drawText("tileset author", 8, 288, 16.0f, 1.0f, 1.0f, 1.0f, alpha); 
		d->drawText(tileSet->getAuthor(), 8, 304, 16.0f, 1.0f, 1.0f, 1.0f, alpha); 
	}

	if(paused)
	{
		d->fillRect(0, 0, 224, 320, 0, 0, 0, 128);

		d->drawText("paused", 50, 40, 32.0f);

		d->drawText("esc to quit", 57, 200, 16.0f);
		d->drawText("fire to resume", 42, 218, 16.0f);
	}
	else if(levelComplete)
	{
		d->drawText("level", 60, 80, 32.0f);
		d->drawText("complete", 30, 112, 32.0f);

		if(matchedTiles == 70)
			d->drawText("perfect bonus", 47, 200, 16.0f, 1.0f, 1.0f, 0.0f);
		if(oneTileOnly)
			d->drawText("one tile bonus", 40, 218, 16.0f, 0.0f, 0.0f, 1.0f);

		d->drawText("press fire", 60, 300, 16.0f);
	}
	else if(gameOver)
	{
		d->drawText("game", 75, 80, 32.0f);
		d->drawText("over", 75, 112, 32.0f);
		d->drawText("press fire", 60, 300, 16.0f);
	}
}

//------------------------------------------------------------------------------
void ShooterGamePlay::keyDown(const SDL_keysym& keyPressed)
{
	if(keyPressed.sym == SDLK_ESCAPE)
	{
		if(!paused)
			paused = true;
		else
			nextState("TitleScreen");
	}
	else if(!levelComplete && !gameOver)
	{
		if(keyPressed.sym == SDLK_LEFT)
			player.moveLeft(true);
		else if(keyPressed.sym == SDLK_RIGHT)
			player.moveRight(true);
		else if(keyPressed.sym == SDLK_UP)
			player.moveUp(true);
		else if(keyPressed.sym == SDLK_DOWN)
			player.moveDown(true);
		else if(keyPressed.sym == SDLK_z)
		{
			if(paused)
				paused = false;
			else
			{
				if(startTimer > 30)
					startTimer = 30;
				fireIsDown = true;
			}
		}
	}
	else if(levelComplete && (keyPressed.sym == SDLK_z))
		nextState("GauntletGamePlay", map, tileSet);
	else if(gameOver && (keyPressed.sym == SDLK_z))
		nextState("FailScreen");
}

//------------------------------------------------------------------------------
void ShooterGamePlay::keyUp(const SDL_keysym& keyPressed)
{
	if(keyPressed.sym == SDLK_LEFT)
		player.moveLeft(false);
	else if(keyPressed.sym == SDLK_RIGHT)
		player.moveRight(false);
	else if(keyPressed.sym == SDLK_UP)
		player.moveUp(false);
	else if(keyPressed.sym == SDLK_DOWN)
		player.moveDown(false);
	else if(keyPressed.sym == SDLK_z)
		fireIsDown = false;
}

//------------------------------------------------------------------------------
void ShooterGamePlay::collideTiles()
{
	int i, j, k;
	float bX, bY;
	float tX, tY;
	float dist;
	int tempint;
	bool hitTile = false;

	for(i=(bullets.size()-1);i>=0;--i)
	{
		bX = bullets[i]->getX()+8.0f;
		bY = bullets[i]->getY()+4.0f;

		for(j=0;j<tiles.size();++j)
		{
			if(tiles[j]->isMoving())
			{
				tX = (tiles[j]->getX()+16.0f) - bX;
				tY = (tiles[j]->getY()+16.0f) - bY;

				dist = sqrtf((tX*tX) + (tY*tY));
				if(dist < 16.0f)
				{
					hitTile = true;

					//Get rid of the bullet.
					delete bullets[i];
					bullets.erase(bullets.begin()+i);

					Sounds::getInstance().playSound(Sounds::TileHit);

					//Get the tile to move to its correct position.
					tX = currentTile % MapWidth;
					tX *= 32;
					tY = currentTile/MapWidth;
					tY *= 32;
					tiles[j]->moveTo(tX, tY);

					if(map[currentTile] == tiles[j]->getIndex())
					{
						GameCounter::getInstance().incrementCount("Matched Tile", 1000);
						++matchedTiles;
					}
					else
					{
						int tempint = tiles[j]->getIndex();

						GameCounter::getInstance().incrementCount("Unmatched Tile", 10);
					}

					map[currentTile] = tiles[j]->getIndex();

					//Update currentTile.
					--currentTile;
					while((map[currentTile] > 191) && (currentTile >= 0))
					{
						++matchedTiles;
						--currentTile;
					}
					if(currentTile < 0)
					{
						currentTile = 0; //Level Complete.
						levelComplete = true;
						GameCounter::getInstance().incrementCount("Shooter Level Complete", 10000);
						if(matchedTiles == 70)
							GameCounter::getInstance().incrementCount("Perfect Match", 10000);

						//Work out if the player got the one-tile-only bonus.
						tempint = map[0];
						for(k=1;k<(MapWidth*MapHeight);++k)
						{
							if(map[k] != tempint)
								break;
						}
						if(k == (MapWidth*MapHeight))
						{
							oneTileOnly = true;
							GameCounter::getInstance().incrementCount("One Tile Only", 10000);
						}
					}

					break;
				}
			}
		}

		if(hitTile)
			continue;

		//Test if an enemy's bullet hit the player.
		if(!hitTile && bullets[i]->getEnemy())
		{
			tX = player.getX() + 8.0f;
			tY = player.getY() + 12.0f;
			tX -= bX;
			tY -= bY;

			dist = sqrtf((tX*tX) + (tY*tY));
			if(dist < 8.0f)
			{
				//Get rid of the bullet.
				delete bullets[i];
				bullets.erase(bullets.begin()+i);

				Sounds::getInstance().playSound(Sounds::ShipDie);

				explosions.push_back(new Explosion(player.getX(), player.getY()));

				if(player.decrementLives())
					gameOver = true;
				else
					Sounds::getInstance().playSound(Sounds::ShipNew);
			}
		}
		//Test if the player's bullet hit an enemy.
		else if(!hitTile)
		{
			for(j=0;j<enemies.size();++j)
			{
				if(enemies[j]->isWarping())
					continue;

				tX = enemies[j]->getX() + 16.0f;
				tY = enemies[j]->getY() + 16.0f;
				tX -= bX;
				tY -= bY;

				dist = sqrtf((tX*tX) + (tY*tY));
				if(dist < 16.0f)
				{
					//Add some particles.
					for(k=0;k<NumParticles;++k)
					{
						if(!particles[k].isActive())
						{
							particles[k].setColour(1.0f, 0.0f, 0.0f);
							particles[k].setSize(6.0f, 6.0f);
							particles[k].setDensity(0.4f);
							particles[k].setDirection(0.1f, 0.5f);
							particles[k].setPosition(bullets[i]->getX()+8.0f, bullets[i]->getY()-4.0f);
							particles[k].start(15);
							break;
						}
					}

					//Get rid of the bullet.
					delete bullets[i];
					bullets.erase(bullets.begin()+i);

					if(enemies[j]->decrementLives())
					{
						Sounds::getInstance().playSound(Sounds::SkeletonDie);
						explosions.push_back(new Explosion(enemies[j]->getX(), enemies[j]->getY()));

						GameCounter::getInstance().incrementCount("Enemy Killed", 500);

						delete enemies[j];
						enemies.erase(enemies.begin()+j);
					}
					else
						Sounds::getInstance().playSound(Sounds::BulletHit);
				}
			}
		}
	}
}

//------------------------------------------------------------------------------
void ShooterGamePlay::collidePlayer()
{
	int i;
	float dist;
	float xP, yP;
	float xT, yT;

	xP = player.getX() + 8.0f;
	yP = player.getY() + 12.0f;

	for(i=0;i<tiles.size();++i)
	{
		if(tiles[i]->isMoving())
		{
			xT = tiles[i]->getX()+16.0f;
			yT = tiles[i]->getY()+16.0f;

			xT -= xP;
			yT -= yP;

			dist = sqrtf((xT*xT) + (yT*yT));
			if(dist < 24.0f)
			{
				delete tiles[i];
				tiles.erase(tiles.begin()+i);

				Sounds::getInstance().playSound(Sounds::ShipDie);

				explosions.push_back(new Explosion(xP-16, yP-16));

				if(player.decrementLives())
					gameOver = true;
				else
					Sounds::getInstance().playSound(Sounds::ShipNew);

				break;
			}
		}
	}
}

//------------------------------------------------------------------------------
void ShooterGamePlay::load()
{
	int i;
	FilePath levelsPath = FilePath::getDataFilesDir().getChild("levels");

	DebugStatement("ShooterGamePlay::load()");

	if(!levelsPath.exists())
		levelsPath.createAsDirectory();

	FilePath filepath = levelsPath.getChild(levelIndex);

	DebugStatement("ShooterGamePlay::load()  Filepath setup:");
	DebugStatement(filepath.getPath());

	//Load the file.
	{
		char buf;
		ifstream phil(filepath.getPath().c_str(), ios::in|ios::binary);

		if(phil.good())
		{
			int numGround, numWall, numImpassable;

			phil.read(&buf, 1);
			tileSet = TileSetFactory::getInstance().getTileSet((uint8_t)buf);

			numGround = tileSet->getNumGroundTiles();
			numWall = tileSet->getNumWallTiles();
			numImpassable = tileSet->getNumImpassableTiles();
			for(i=0;i<(MapWidth*MapHeight);++i)
			{
				phil.read(&buf, 1);
				map[i] = (uint8_t)buf;
				if(map[i] < 63)
				{
					assert(map[i] < numGround);
					if(map[i] >= numGround)
					{
						DebugStatement("Ground tile out of range:");
						DebugStatement(map[i]);
						map[i] = 0;
					}
				}
				else if(map[i] < 128)
				{
					assert((map[i]-64) < numWall);
					if((map[i]-64) >= numWall)
					{
						DebugStatement("Wall tile out of range:");
						DebugStatement(map[i]);
						map[i] = 0;
					}
				}
				else if(map[i] < 192)
				{
					assert((map[i]-128) < numImpassable);
					if((map[i]-128) >= numImpassable)
					{
						DebugStatement("Impassable tile out of range:");
						DebugStatement(map[i]);
						map[i] = 0;
					}
				}
				else
				{
					assert(map[i] == 192);
					if(map[i] > 192)
					{
						DebugStatement("Spawner tile out of range:");
						DebugStatement(map[i]);
						map[i] = 0;
					}
				}
			}
		}
		else
			cout << "ShooterGamePlay: Could not load level file: " << filepath.getPath() << endl;
	}
}
